function [pred,err] =  NOCtidepred(mjd,ha,ga,varargin)
% [pred,err] =  NOCtidepred(mjd,ha,ga,varargin)
%
%==========================================================================================
%
% produces npred tide predictions at times given by mjd (modified julian date),
% given a list amplitudes (ha) and greenwich phase lags (ga). The constituents
% can be defined in a number of ways
%
% INPUT:
%
%   mjd (double array length npred) - modified julian day 
%    ha (double array,     0:ncmax) - amplitudes of constituents (1:ndc+nrel)
%                                     amplitude of constant offset (0)
%    ga (double array length ncmax) - greenwich phases of constituents
%                                     (degrees)
%
% If no other parameters are supplied then the program will assume, if it can,  an
% appropriate set of tides based on the number of input phases and amplitudes (ncmax) 
%
% If you wish to tell the program what harmonics to use then you can either
% (1) specify a "ctl" file using 'filename'
% (2) input a structure, S, consisting of parts sig, sigrel, relamp, and relpha
%
%      sig (double array len ndc+nrel)  - speeds of primary (1:ndc) and
%                                         related (ndc+1,ndc+nrel)constituents
%    sigrel(double array length nrel)   - (n)=speed of primary constituent
%                                         related to related constituent
%                                         with speed sig(ndc+n)
%    relamp(double array length nrel)   - related/primary amplitude
%    relpha(double array length nrel)   - related-primary phase
%
% If sigrel is not empty then an 
% extra set of tides are included without increasing the number of
% fitted parameters, by assuming a constant amplitude factor (relamp)
% and relative phase (relpha) in comparison with one of the fitted
% constituents.  Most common is for relamp to reflect relative
% amplitudes of the equilibrium constituents, and relpha=0.
%
% sigrel contains 2 lists of speeds: one for the speed of the
% constituent to be calculated by association, the other for the speed of
% the constituent it is related to.
%
% (3) enter a choice for the harmonics out of the following list using 'force'
%      '18yr' suitable for 18 years of data
%      '1yr' suitable for 1 year of data
%      '1yr_nolong' suitable for 1 year without long period harmonics
%      '6mo' suitable for 6 months of data
%      '6mo_nolong' suitable for 6 months of data without long period harmonics
%      '1mo' suitable for 1 month of data
%
% OPTIONAL ARGUMENTS :
%
% These come in pairs with the name and then the variable e.g. 'verbose','long'
%
% verbose (char)                        - Output some extra information to screen
%                                         Can be either :
%                                         on/yes/short : brief output
%                                         off/no       : no output (DEFAULT)  
%                                         long         : Outputs the constituents and their 
%                                                        estimated amplitude and phase
% constant (char)                       - Choose whether to add Z0 to the series
%                                         Can either be :
%                                         on/yes        : add Z0
%                                         off/no        : don't add Z0 (DEFAULT)
%
%==========================================================================================
%
% OUTPUT:
%
% pred (double array length npred) - predicted tide
% nerr (integer) - error flag
%                  0 = no error
%                  1 = year out of range
%                  2 = speeds (sig) don't match reference list (sig0)
%                      or out of order
%                  3 = end time before start time
%                  4 = ndc > ncmax - too many constituents
%
%==========================================================================================
%
% From Simon Williams, NOC. ~2018

%
%     implicit none
%     integer ncmax,idmin,ndayfu
%     parameter(ncmax=120,idmin=-1409059,ndayfu=4)
%     integer mjdns,mjdne,mjdn0,mjdn,npred,ndc,nrel,nerr,i,j,k
%     integer ntab(ncmax)
%     double precision hrs,hre,t,ts,te,tprime,rad
%     double precision sig(ncmax), sig0(ncmax), ha(0:ncmax), ga(ncmax)
%     double precision pred(npred), f(ncmax), v(ncmax)
%
%
%


ncmax  = 120;
idmin  = -1409059;
ndayfu = 30;
rad    = pi/180;
deg    = 1.0 ./ rad;

sig0 = [0.0410686;   0.0821373;   0.5443747;   1.0158958;
        1.0980330;  12.8542862;  12.9271398;  13.3986609;
       13.4715145;  13.9430356;  14.0251729;  14.4920521;
       14.5695476;  14.9178647;  14.9589314;  15.0000000;
       15.0410686;  15.0821353;  15.1232059;  15.5125897;
       15.5854433;  16.0569644;  16.1391017;  27.3416965;
       27.4238338;  27.8953548;  27.9682085;  28.4397295;
       28.5125832;  28.9019670;  28.9841042;  29.0662415;
       29.4556253;  29.5284789;  29.9589333;  30.0000000;
       30.0410667;  30.0821373;  30.5443747;  30.6265120;
       31.0158958;  42.9271398;  43.4761564;  43.9430356;
       44.0251729;  45.0410686;  57.4238338;  57.9682085;
       58.4397295;  58.9841042;  59.0662415;  60.0000000;
       60.0821373;  86.4079380;  86.9523127;  87.4238338;
       87.9682085;  88.0503458;  88.9841042;  89.0662415;
       26.4079380;  26.8701754;  26.9523127;  27.5059711;
       28.3575923;  29.9178627;  31.0887494;  42.3827651;
       43.0092771;  44.5695476;  56.8701754;  56.9523127;
       57.8860712;  71.9112441;  72.4602606;  73.0092771;
       84.8476675;  85.3920423;  85.8542797;  85.9364170;
       86.3258007;  86.4807917;  86.8701754;  87.4966874;
       88.5125832;  88.5947205; 114.8476675; 115.3920423;
      115.9364170; 116.4079380; 116.9523127; 117.0344500;
      117.5059711; 117.9682085; 118.0503458; 145.9364170;
      146.9523127; 174.3761465; 174.9205212; 175.9364170;
       27.4966874;  27.8860712;  28.9430356;  29.0251729;
       30.4715211;  31.0980330;  56.4079380;  57.4966874;
       58.5125832;  59.5284789;  28.3986629;  28.4807962;
       72.9271398;  74.0251729;  29.5284789;   0.0000000;
        0.0000000;   0.0000000;   0.0000000;   0.0000000];


names = {'SA';   'SSA'; 'MM'; 'MSF'; 'MF'; '2Q1'; 'SIG1'; 'Q1'; 'RO1';
         'O1';   'MP1'; 'M1'; 'CHI1'; 'PI1'; 'P1'; 'S1'; 'K1'; 'PSI1'; 'PHI1';
         'TH1';  'J1'; 'SO1'; 'OO1'; 'OQ2'; 'MNS2'; '2N2'; 'MU2'; 'N2'; 'NU2';
         'OP2';  'M2'; 'MKS2'; 'LAM2'; 'L2'; 'T2'; 'S2'; 'R2'; 'K2'; 'MSN2';
         'KJ2';  '2SM2'; 'MO3'; 'M3'; 'SO3'; 'MK3'; 'SK3'; 'MN4'; 'M4'; 'SN4';
         'MS4';  'MK4'; 'S4'; 'SK4'; '2MN6'; 'M6'; 'MSN6'; '2MS6'; '2MK6'; '2SM6';
         'MSK6'; '2MN2S2'; '3MSK2'; '3M2S2'; 'MNK2S2'; 'SNK2'; '2SK2'; '2MS2N2';
         'MQ3';  '2MP3'; '2MQ3'; '3MK4'; '3MS4'; '2MSK4'; '3MK5'; 'M5'; '3MO5';
         '2MNS6';'3MNS6'; '4MK6'; '4MS6'; '2MSNK6'; '2MV6'; '3MSK6'; '4MN6'; '3MSN6';
         'MKL6'; '2MN8'; '3MN8'; 'M8'; '2MSN8'; '3MS8'; '3MK8'; 'MSNK8'; '2MS8';
         '2MSK8';'4MS10'; '3M2S10'; '4MSN12'; '5MS12'; '4M2S12'; 'MVS2'; '2MK2';
         'MA2';  'MB2'; 'MSV2'; 'SKM2'; '2MNS4'; 'MV4'; '3MN4'; '2MSN4'; 'NA2';
         'NB2';  'MSO5'; 'MSK5'; '2MN2'};

     %Quick work-around for empty prediction. 
if isempty(ga)
    pred=nan(size(mjd));
    err=[];
    return
end
     
if mod(nargin,2) == 0
        error('Incorrect number of input arguments : varargin arguments need to be in pairs.');
end

nconstituents = length(ga);

rowcheck = 0;

if isrow(mjd)
	rowcheck = 1;
	mjd = mjd';
end

if length(ha) ~= nconstituents+1
	error('Input amplitudes (ha) is not consistent with the number of input phases (one more)');
end 

gotCtl  = 0;
verbose = 0;
addZ0   = 0;

for j = 1:2:nargin-3
        switch lower(varargin{j})
                case 'filename'
                        filename = varargin{j+1};
                        validateattributes(filename,{'char'},{'nonempty'});
                        [sig,sigrel,relamp,relpha,ctl_names] = readctl(filename);
			nrel = length(sigrel);
			ndc = length(sig) - nrel;
                        gotCtl = 1;
                case 'struct'
                        S = varargin{j+1};
                        if ~isstruct(S)
                                error(sprintf('struct %s is not a structure', inputname(j+1)));
                        end
                        sig = S.sig;
                        sigrel = S.sigrel;
                        relamp = S.relamp;
                        relpha = S.relpha;
                        ctl_names = S.names;
			nrel = length(sigrel);
			ndc = length(sig) - nrel;
                        gotCtl = 1;
                case 'force'
                        force_ctl = varargin{j+1};
                        validateattributes(force_ctl,{'char'},{'nonempty'});
                        gotCtl = -1;
                case 'verbose'
                        verbose_cmd = varargin{j+1};
                        validateattributes(verbose_cmd,{'char'},{'nonempty'});
                        switch lower(verbose_cmd)
                                case 'on'
                                        verbose = 1;
                                case 'yes'
                                        verbose = 1;
                                case 'no'
                                        verbose = 0;
                                case 'off'
                                        verbose = 0;
                                case 'long';
                                        verbose = 2;
                                case 'short'
                                        verbose = 1;
                                otherwise
                                        error(sprintf('Unknown verbose option %s', verbose_cmd))
                        end
                case 'constant'
                        constant_cmd = varargin{j+1};
                        validateattributes(constant_cmd,{'char'},{'nonempty'});
                        switch lower(constant_cmd)
                                case 'on'
                                        addZ0 = 1;
                                case 'yes'
                                        addZ0 = 1;
                                case 'no'
                                        addZ0 = 0;
                                case 'off'
                                        addZ0 = 0;
                                otherwise
                                        error(sprintf('Unknown constant option %s', constant_cmd))
                        end
                otherwise
                        error(sprintf('Unknown input argument : %s',varargin{j}))
        end
end

if gotCtl == 0
	% We are going to try to guess from the number of constituents
	% If we can't then will output an error
	switch nconstituents
		case 115
			force_ctl = '18yr';
		case 34
			force_ctl = '1mo';
		case 62
			force_ctl = '1yr';
		case 57
			force_ctl = '1yr_nolong';
		case 56
			force_ctl = '6mo';
		case 52
			force_ctl = '6mo_nolong';
		otherwise
			error('Unkown number of constituents : cannot guess constituent set');
	end
	if verbose > 0
		disp(sprintf('Number of constituents indicates you are using %s',force_ctl))
	end
end

if gotCtl <= 0
        switch lower(force_ctl)
                case '18yr'
                        p = (1:115)';
                        sig = sig0(p);
                        sigrel = [];
                        relamp = [];
                        relpha = [];
                        ctl_names = names(p);
                case '1yr'
                        p = find(logical([ones(60,1);zeros(42,1);1;1;zeros(11,1)]));
                        sig = sig0(p);
                        sigrel = [];
                        relamp = [];
                        relpha = [];
                        ctl_names = names(p);
                case '1yr_nolong'
                        p = find(logical([zeros(5,1);ones(55,1);zeros(42,1);1;1;zeros(11,1)]));
                        sig = sig0(p);
                        sigrel = [];
                        relamp = [];
                        relpha = [];
                        ctl_names = names(p);
                case '6mo'
                        p       = [2;3;4;5;6;7;8;9;10;11;12;13;15;17;19;20;21;22;23;24;25;26;27;28;29;30;31;32;33;34;36;38;39;40;41;42;43;44;45;46;47;48;49;50;51;52;53;54;55;56;57;58;59;60;14;35];
                        sig = sig0(p);
                        sigrel  = sig0([17;36]);
                        relamp  = [0.019;0.059];
                        relpha  = [0.0;0.0];
                        ctl_names = names(p);
                case '6mo_nolong'
                        p       = [6;7;8;9;10;11;12;13;15;17;19;20;21;22;23;24;25;26;27;28;29;30;31;32;33;34;36;38;39;40;41;42;43;44;45;46;47;48;49;50;51;52;53;54;55;56;57;58;59;60;14;35];
                        sig = sig0(p);
                        sigrel  = sig0([17;36]);
                        relamp  = [0.019;0.059];
                        relpha  = [0.0;0.0];
                        ctl_names = names(p);
                case '1mo'
                        p = [3;4;8;10;12;17;21;23;27;28;31;34;36;41;42;43;45;47;48;49;50;54;55;56;57;59;14;15;18;19;26;29;35;38];

                        sig    = sig0(p);
                        sigrel = sig0([17;17;17;17;28;28;36;36]);
                        relamp = [0.019;0.331;0.008;0.014;0.133;0.194;0.059;0.272];
                        relpha =  zeros(8,1);
                        ctl_names = names(p);
                otherwise
                        error(sprintf('unknown choice of harmonics : %s', force_ctl))

        end
	nrel = length(sigrel);
	ndc = length(sig) - nrel;
end


% Start of the main program

nerr = 0;

npred = length(mjd);
mjdns = floor(mjd);
hrs = (mjd - mjdns) * 24;

pred = zeros(npred,1);


if any(mjdns < idmin)
        nerr = 1;
        return
end

if(ndc+nrel >= ncmax  |  ndc < 0 |  nrel < 0)
	nerr = 4;
	return
end

%
% compile a table matching speeds in sig with those in sig0
%

if any(sig == 0)
        nerr = 2;
        return
end


ntab = zeros(ndc+nrel,1);

j = 1;
for i = 1:ndc
        while abs(sig(i)-sig0(j)) > 0.00001
                j = j + 1;
                if (j > ncmax)
                        nerr = 2;
                        return
                end
        end
        ntab(i) = j;
end

j = 1;
for i = ndc+1:ndc+nrel
        while abs(sig(i)-sig0(j)) > 0.00001
                j = j + 1;
                if (j > ncmax)
                        nerr = 2;
                        return
                end
        end
        ntab(i) = j;
end

%
% Loop through chosen times calculating predictions. Reference
% phases and nodal amplitude factors are calculated at 00:00
% on day mjdn0. mjdn0 is chosen to be within ndayfu/2 days of
% prediction time, on a day with mjdn0 divisible by ndayfu.
% mjdn0 is changed only when prediction time is incremented past
% day mjdn0+ndayfu/2. Time measured relative to reference time
% is tprime (days)
%
[f,v] = phamp0fast(mjdns);

pred = zeros(npred,1);

%
% Sum constant + constituents to give prediction
%

for j = 1:ndc+nrel
	k = ntab(j);
	pred = pred + ha(j+1).*f(:,k).*cos(rad.*( sig0(k)*hrs + v(:,k) - ga(j) ));
end

if addZ0 == 1
	pred = pred + ha(1);
end

if rowcheck
	pred = pred';
end

end
