function [chan60flagSIMcell,chanflagSIMcell]=channelsimilar(chanvec,...
    dates60cell,heights60cell,...
    datescell,heightscell,...
    datumcor,FlagChannelDiffGreaterThan)

% CHANNELSIMILAR : = Identify massive deviations between sensors
% 
% Return the channels that are similar - 
% chanSIM=1(true) for SIMILAR to another,  0(false) for DIFFERENT
%
% This function is required for TIDEGAUGEQC.
% 
% Joanne Williams, joll@noc.ac.uk, Oct 2018


%% Look for massive deviations between sensors. This might pick up a sensor that has failed in a surprising way

%CALQ 2014 is an example where there is a big deviation in the signal that
%looks highly suspect.
%
% This has to be done *before* tidal analysis (ie not on Non-tidal-resid)
% as bad deviations will tend to be spread by the de-tiding, like filter
% fringes.
%
%(1) Check whether the two signals look alike. (hourly NTR)
%NO! - NTR is no good as big problems will spread into wider time periods by
%the tidal fitting. So we need to compare the raw signals (or maybe hourly,
%to reduce difference in sampling between sensors)
%
%How do we identify where they are different? Are there legitimate reasons
%for them to differ?
% Find where they are"very" different. In window around difference, flag
% both.
% Do first pass tidal fit & pred ignoring flagged data.
% Keep the channel that is closer to the tidal pred?

% In the event of a constant offset between channels the tendency will be
% to keep most data.

if length(chanvec)==1
    warning('Only one channel - no channel comparison performed')
    disp('Beware that severe channel errors may pass QC - visual inspection recommended')
    chan60flagSIM1=true(size(dates60cell{chanvec(1)}));
    
    chan60flagSIMcell{chanvec(1)}=chan60flagSIM1;
    chanflagSIMcell{chanvec(1)}=true(size(datescell{chanvec(1)}));
    
elseif length(chanvec)==2
    %2 channels compared
    
    
    dates60_1=dates60cell{chanvec(1)};
    dates60_2=dates60cell{chanvec(2)};
    
    heights60_1=heights60cell{chanvec(1)};
    heights60_2=heights60cell{chanvec(2)};
    
    % medh_1=medianheightscell{chanvec(1)};
    % medh_2=medianheightscell{chanvec(2)};
    
    N601=length(dates60_1);
    N602=length(dates60_2);
    
    %Find the intersect of dates between 2 of the channels.
    %iA_21 is index into channel 2 of channel 1 etc
    [~,iA_21,iA_12] = intersect(dates60_2,dates60_1);
    
    %Compare heights *only* for these dates
    %IN HERE - could ADD CORRECTION FOR TREND?
    chan60diff21=heights60_2(iA_21)-heights60_1(iA_12)...
        -(datumcor{chanvec(2)}-datumcor{chanvec(1)}); %allow for constant offset between channels
    
    %Define how much difference we think is reasonable to declare the same
    chan60flagSIM21=(abs(chan60diff21)<4*median(abs(chan60diff21),'omitnan')); %1 for nearly same, 0 for suspicious or one is NAN, but there should be no nans!
    
    %Hard-coded Back-up. Hourly Channel differences greater than 10cm after
    %instrument offset are flagged for comparison.
    %FlagChannelDiffGreaterThan=0.1 (hard code above)
    
    chan60flagSIM21(abs(chan60diff21)>FlagChannelDiffGreaterThan)=0;
    
    
    
    %For each channel, flag as SIM if it is similar to at least one of the others,
    %Also mark it as SIM here if there's no information because there's only
    %one channel for a while (easier with just 2 channels!)
    chan60flagSIM1=ones(N601,1);
    chan60flagSIM2=ones(N602,1);
    
    chan60flagSIM1(iA_12)=chan60flagSIM21; %is 1 similar to 2? (for comparable dates)
    chan60flagSIM2(iA_21)=chan60flagSIM21; %is 2 similar to 1? (for comparable dates)
    
    %For each channel in turn,
    %assume data between nearby dissimilar points is also dissimilar
    dis1=find(~chan60flagSIM1);
    disdates1=dates60_1(~chan60flagSIM1);
    idisdatesnear=find(diff(disdates1)<1); %index of disdates closer than 1 day to next disdate
    for i=1:length(idisdatesnear)
        chan60flagSIM1(dis1(idisdatesnear(i)):dis1(idisdatesnear(i)+1))=0;
    end
    
    dis2=find(~chan60flagSIM2);
    disdates2=dates60_2(~chan60flagSIM2);
    idisdatesnear=find(diff(disdates2)<1); %index of disdates closer than 1 day to next disdate
    for i=1:length(idisdatesnear)
        chan60flagSIM2(dis2(idisdatesnear(i)):dis2(idisdatesnear(i)+1))=0;
    end
    
    %     %(For testing) Shade the suspect sections from badON to badOFF
    %     patchbackgroundx(dates60_1(diff(chan60flagSIM1)==-1),dates60_1(diff(chan60flagSIM1)==1),'facecolor','m','edgealpha',0.1,'facealpha',0.5);
    %     patchbackgroundx(dates60_2(diff(chan60flagSIM2)==-1),dates60_2(diff(chan60flagSIM2)==1),'facecolor','c','edgealpha',0.1,'facealpha',0.5);
    %
    %When spreading to everywhere in the hour, we have to be extra careful
    %because dates60_1 is gappy.
    %So spread to remove gaps
    dates60_1_full=(dates60_1(1):1/24:dates60_1(end))';
    chan60flagSIM1_full=true(size(dates60_1_full));
    chan60flagSIM1_full(ismembertol(dates60_1_full,dates60_1))=chan60flagSIM1; %use tol to ensure no floating point problems here
    dates60_2_full=(dates60_2(1):1/24:dates60_2(end))';
    chan60flagSIM2_full=true(size(dates60_2_full));
    chan60flagSIM2_full(ismembertol(dates60_2_full,dates60_2))=chan60flagSIM2; %use tol to ensure no floating point problems here
    
    
    chan60flagSIMcell{chanvec(1)}=chan60flagSIM1;
    chan60flagSIMcell{chanvec(2)}=chan60flagSIM2;
    
    
    
    %Then Spread this to everywhere in that hour. Extrapolate with true as
    %that's where there's no channel comparison available.
    
    %Still assumed to be SIM where no channel comparison was possible
    chanflagSIMcell{chanvec(1)}=logical(interp1(dates60_1_full,single(chan60flagSIM1_full),datescell{chanvec(1)},'nearest',1));
    chanflagSIMcell{chanvec(2)}=logical(interp1(dates60_2_full,single(chan60flagSIM2_full),datescell{chanvec(2)},'nearest',1));
    
    
    
    
    %For each channel in turn, assume data between nearby suspect points is
    %also suspect (This may not be necessary as done above?)
    dis1=find(~chanflagSIMcell{chanvec(1)});
    disdates1=datescell{chanvec(1)}(~chanflagSIMcell{chanvec(1)});
    idisdatesnear=find(diff(disdates1)<1); %index of disdates closer than 1 day to next disdate
    for i=1:length(idisdatesnear)
        chanflagSIMcell{chanvec(1)}(dis1(idisdatesnear(i)):dis1(idisdatesnear(i)+1))=0;
    end
    
    dis2=find(~chanflagSIMcell{chanvec(2)});
    disdates2=datescell{chanvec(2)}(~chanflagSIMcell{chanvec(2)});
    idisdatesnear=find(diff(disdates2)<1); %index of disdates closer than 1 day to next disdate
    for i=1:length(idisdatesnear)
        chanflagSIMcell{chanvec(2)}(dis2(idisdatesnear(i)):dis2(idisdatesnear(i)+1))=0;
    end
    
    
    %If first suspect point is within 1 day of the beginning, assume all the first
    %day is suspect
    if (disdates1(1)-datescell{chanvec(1)}(1))<1
        chanflagSIMcell{chanvec(1)}(1:dis1(1))=0;
    end
    if (disdates2(1)-datescell{chanvec(2)}(1))<1
        chanflagSIMcell{chanvec(2)}(1:dis2(1))=0;
    end
    
    %If last suspect point is within 1 day of the end, assume all the last
    %day is suspect
    if (datescell{chanvec(1)}(end)-disdates1(end))<1
        chanflagSIMcell{chanvec(1)}(dis1(end):end)=0;
    end
    if (datescell{chanvec(2)}(end)-disdates2(end))<1
        chanflagSIMcell{chanvec(2)}(dis2(end):end)=0;
    end
    
    %     %Shade the suspect sections from badON to badOFF
    %     patchbackgroundx(datescell{chanvec(1)}(diff(chanflagSIMcell{chanvec(1)})==-1),datescell{chanvec(1)}(diff(chanflagSIMcell{chanvec(1)})==1),'facecolor','r','edgecolor','none','facealpha',0.5);
    %     patchbackgroundx(datescell{chanvec(2)}(diff(chanflagSIMcell{chanvec(2)})==-1),datescell{chanvec(2)}(diff(chanflagSIMcell{chanvec(2)})==1),'facecolor','b','edgecolor','none','facealpha',0.5);
    %
elseif length(chanvec)==3
    %3 channels compared
    
    dates60_1=dates60cell{chanvec(1)};
    dates60_2=dates60cell{chanvec(2)};
    dates60_3=dates60cell{chanvec(3)};
    
    heights60_1=heights60cell{chanvec(1)};
    heights60_2=heights60cell{chanvec(2)};
    heights60_3=heights60cell{chanvec(3)};
    
    %     medh_1=medianheightscell{chanvec(1)};
    %     medh_2=medianheightscell{chanvec(2)};
    %     medh_3=medianheightscell{chanvec(3)};
    
    
    N601=length(dates60_1);
    N602=length(dates60_2);
    N603=length(dates60_3);
    
    %Find the intersect of dates between 2 of the channels.
    %iA_21 is index into channel 2 of channel 1 etc
    [~,iA_21,iA_12] = intersect(dates60_2,dates60_1);
    [~,iA_31,iA_13] = intersect(dates60_3,dates60_1);
    [~,iA_32,iA_23] = intersect(dates60_3,dates60_2);
    iA_1only=setxor(1:N601,union(iA_12,iA_13)); %index into channel 1 that is not in any other channel
    iA_2only=setxor(1:N602,union(iA_21,iA_23)); %index into channel 2 that is not in any other channel
    iA_3only=setxor(1:N603,union(iA_31,iA_32)); %index into channel 3 that is not in any other channel
    
    %Compare heights *only* for these dates
    %Constant offset allowed for.
    %Could allow for trend?
    chan60diff21=heights60_2(iA_21)-heights60_1(iA_12)-(datumcor{chanvec(2)}-datumcor{chanvec(1)});
    chan60diff31=heights60_3(iA_31)-heights60_1(iA_13)-(datumcor{chanvec(3)}-datumcor{chanvec(1)});
    chan60diff32=heights60_3(iA_32)-heights60_2(iA_23)-(datumcor{chanvec(3)}-datumcor{chanvec(2)});
    
    
    %Define how much difference we think is reasonable. There should be no nans!
    
    chan60flagSIM21=(abs(chan60diff21)<4*median(abs(chan60diff21),'omitnan')); %1 for nearly same, 0 for suspicious or one is NAN
    chan60flagSIM31=(abs(chan60diff31)<4*median(abs(chan60diff31),'omitnan')); %1 for nearly same, 0 for suspicious or one is NAN
    chan60flagSIM32=(abs(chan60diff32)<4*median(abs(chan60diff32),'omitnan')); %1 for nearly same, 0 for suspicious or one is NAN
    
    %This method assumes that there is a long period for which there should
    %be consistent channels. If one is increasingly bad right through, it
    %might get missed. Hence...
    
    %Hard-coded Back-up. Hourly Channel differences greater than 10cm after
    %instrument offset are flagged for comparison.
    %FlagChannelDiffGreaterThan=0.1 % hard code at start of file.
    chan60flagSIM21(abs(chan60diff21)>FlagChannelDiffGreaterThan)=0;
    chan60flagSIM32(abs(chan60diff32)>FlagChannelDiffGreaterThan)=0;
    chan60flagSIM31(abs(chan60diff31)>FlagChannelDiffGreaterThan)=0;
    
    %For each channel, flag as SIM if it is similar to at least one of the others,
    %Also mark it as SIM here if there's no information because there's only
    %one channel for a while
    
    chan60flagSIM1=ones(N601,1);
    chan60flagSIM2=ones(N602,1);
    chan60flagSIM3=ones(N603,1);
    
    chan60flagSIM1(iA_1only)=1; %index into 1 that is NOT in any other channel
    chan60flagSIM1(iA_12)=chan60flagSIM21; %is 1 similar to 2? (for comparable dates)
    chan60flagSIM1(iA_13)=chan60flagSIM31|chan60flagSIM1(iA_13); %Or is 1 similar to 3? (for comparable dates)
    
    chan60flagSIM2(iA_2only)=1;
    chan60flagSIM2(iA_23)=chan60flagSIM32;
    chan60flagSIM2(iA_21)=chan60flagSIM21|chan60flagSIM2(iA_21);
    
    chan60flagSIM3(iA_3only)=1;
    chan60flagSIM3(iA_31)=chan60flagSIM31;
    chan60flagSIM3(iA_32)=chan60flagSIM32|chan60flagSIM3(iA_32);
    
    
    %For each channel in turn,
    %assume data between nearby dissimilar points is also dissimilar (Eg If
    %there's a potential gauge problem, we don't trust the data for a while)
    dis1=find(~chan60flagSIM1);
    disdates1=dates60_1(~chan60flagSIM1);
    idisdatesnear=find(diff(disdates1)<1); %index of disdates closer than 1 day to next disdate
    for i=1:length(idisdatesnear)
        chan60flagSIM1(dis1(idisdatesnear(i)):dis1(idisdatesnear(i)+1))=0;
    end
    
    dis2=find(~chan60flagSIM2);
    disdates2=dates60_2(~chan60flagSIM2);
    idisdatesnear=find(diff(disdates2)<1); %index of disdates closer than 1 day to next disdate
    for i=1:length(idisdatesnear)
        chan60flagSIM2(dis2(idisdatesnear(i)):dis2(idisdatesnear(i)+1))=0;
    end
    
    dis3=find(~chan60flagSIM3);
    disdates3=dates60_3(~chan60flagSIM3);
    idisdatesnear=find(diff(disdates3)<1); %index of disdates closer than 1 day to next disdate
    for i=1:length(idisdatesnear)
        chan60flagSIM3(dis3(idisdatesnear(i)):dis3(idisdatesnear(i)+1))=0;
    end
    
    
    chan60flagSIMcell{chanvec(1)}=chan60flagSIM1;
    chan60flagSIMcell{chanvec(2)}=chan60flagSIM2;
    chan60flagSIMcell{chanvec(3)}=chan60flagSIM3;
    
    
    %     %Shade the suspect sections, where chan60flag=0
    %     %(plot looks for switch beteen states)
    %     patchbackgroundx(dates60_1(diff(chan60flagSIM1)==-1),dates60_1(diff(chan60flagSIM1)==1),'facecolor','m','edgealpha',0.1,'facealpha',0.5);
    %     patchbackgroundx(dates60_2(diff(chan60flagSIM2)==-1),dates60_2(diff(chan60flagSIM2)==1),'facecolor','c','edgealpha',0.1,'facealpha',0.5);
    %     patchbackgroundx(dates60_3(diff(chan60flagSIM3)==-1),dates60_3(diff(chan60flagSIM3)==1),'facecolor','y','edgealpha',0.1,'facealpha',0.5);
    %
    %Spread this to everywhere in that hour.
    
    % we have to be extra careful because dates60_1 is gappy.
    %So first spread to remove gaps
    dates60_1_full=(dates60_1(1):1/24:dates60_1(end))';
    chan60flagSIM1_full=true(size(dates60_1_full));
    chan60flagSIM1_full(ismembertol(dates60_1_full,dates60_1))=chan60flagSIM1; %use tol to ensure no floating point problems here
    
    dates60_2_full=(dates60_2(1):1/24:dates60_2(end))';
    chan60flagSIM2_full=true(size(dates60_2_full));
    chan60flagSIM2_full(ismembertol(dates60_2_full,dates60_2))=chan60flagSIM2; %use tol to ensure no floating point problems here
    
    dates60_3_full=(dates60_3(1):1/24:dates60_3(end))';
    chan60flagSIM3_full=true(size(dates60_3_full));
    chan60flagSIM3_full(ismembertol(dates60_3_full,dates60_3))=chan60flagSIM3; %use tol to ensure no floating point problems here
    
    %Then Spread this to everywhere in that hour. Extrapolate with true as
    %that's where there's no channel comparison available.
    
    %Still assumed to be SIM where no channel comparison was possible
    chanflagSIMcell{chanvec(1)}=logical(interp1(dates60_1_full,single(chan60flagSIM1_full),datescell{chanvec(1)},'nearest',1));
    chanflagSIMcell{chanvec(2)}=logical(interp1(dates60_2_full,single(chan60flagSIM2_full),datescell{chanvec(2)},'nearest',1));
    chanflagSIMcell{chanvec(3)}=logical(interp1(dates60_3_full,single(chan60flagSIM3_full),datescell{chanvec(3)},'nearest',1));
    
    
    
    %For each channel in turn, assume data between nearby suspect points is
    %also suspect (This may not be necessary as done above?)
    dis1=find(~chanflagSIMcell{chanvec(1)});
    disdates1=datescell{chanvec(1)}(~chanflagSIMcell{chanvec(1)});
    idisdatesnear=find(diff(disdates1)<1); %index of disdates closer than 1 day to next disdate
    for i=1:length(idisdatesnear)
        chanflagSIMcell{chanvec(1)}(dis1(idisdatesnear(i)):dis1(idisdatesnear(i)+1))=0;
    end
    
    dis2=find(~chanflagSIMcell{chanvec(2)});
    disdates2=datescell{chanvec(2)}(~chanflagSIMcell{chanvec(2)});
    idisdatesnear=find(diff(disdates2)<1); %index of disdates closer than 1 day to next disdate
    for i=1:length(idisdatesnear)
        chanflagSIMcell{chanvec(2)}(dis2(idisdatesnear(i)):dis2(idisdatesnear(i)+1))=0;
    end
    
    dis3=find(~chanflagSIMcell{chanvec(3)});
    disdates3=datescell{chanvec(3)}(~chanflagSIMcell{chanvec(3)});
    idisdatesnear=find(diff(disdates3)<1); %index of disdates closer than 1 day to next disdate
    for i=1:length(idisdatesnear)
        chanflagSIMcell{chanvec(3)}(dis3(idisdatesnear(i)):dis3(idisdatesnear(i)+1))=0;
    end
    
    
    %If first suspect point is within 1 day of the beginning, assume all the first
    %day is suspect
    if (disdates1(1)-datescell{chanvec(1)}(1))<1
        chanflagSIMcell{chanvec(1)}(1:dis1(1))=0;
    end
    if (disdates2(1)-datescell{chanvec(2)}(1))<1
        chanflagSIMcell{chanvec(2)}(1:dis2(1))=0;
    end
    if (disdates3(1)-datescell{chanvec(3)}(1))<1
        chanflagSIMcell{chanvec(3)}(1:dis3(1))=0;
    end
    
    %If last suspect point is within 1 day of the end, assume all the last
    %day is suspect
    if (datescell{chanvec(1)}(end)-disdates1(end))<1
        chanflagSIMcell{chanvec(1)}(dis1(end):end)=0;
    end
    if (datescell{chanvec(2)}(end)-disdates2(end))<1
        chanflagSIMcell{chanvec(2)}(dis2(end):end)=0;
    end
    if (datescell{chanvec(3)}(end)-disdates3(end))<1
        chanflagSIMcell{chanvec(3)}(dis3(end):end)=0;
    end
    
    
    
    
    %     %Shade the suspect sections from badON to badOFF
    %     patchbackgroundx(datescell{chanvec(1)}(diff(chanflagSIMcell{chanvec(1)})==-1),datescell{chanvec(1)}(diff(chanflagSIMcell{chanvec(1)})==1),'facecolor','r','edgecolor','none','facealpha',0.5);
    %     patchbackgroundx(datescell{chanvec(2)}(diff(chanflagSIMcell{chanvec(2)})==-1),datescell{chanvec(2)}(diff(chanflagSIMcell{chanvec(2)})==1),'facecolor','b','edgecolor','none','facealpha',0.5);
    %     patchbackgroundx(datescell{chanvec(3)}(diff(chanflagSIMcell{chanvec(3)})==-1),datescell{chanvec(3)}(diff(chanflagSIMcell{chanvec(3)})==1),'facecolor','g','edgecolor','none','facealpha',0.5);
    
    
else %length(chanvec)>3
    error('Comparison test currently not implemented for more than 3 channels.')
    
end
%
