% GETSKEWSURGE Calculate skew surges from sea-level and
% "non-tidal-residual"
% This function is required for TIDEGAUGEQC.
% 
%Joanne Williams, joll@noc.ac.uk, circa 2015

function [skews,maxTide,datesskews,datesmaxTide]=getskewsurge(dates,sealevel,residual)

%Assign missing values to NaNs here to avoid risk of including them later.
%There may be nans (-99) in residual that are not in sealevel.
inans=isnan(sealevel); %[Nt]
inanr=isnan(residual); %[Nt]
slnonans=~(inans|inanr);%[Nt]
sealevel(~slnonans)=nan;%[Nt]
residual(~slnonans)=nan;%[Nt]

predictedtide=sealevel-residual;

Nt=length(residual);
diffdateshr=diff(dates)*24; %interval between timeseries in hours %note [Nt-1]

%%
% How many changes in date spacing have they got? This gives a clue as to
% whether nans are used or dates are missing for bad data.
    %note that montauk at least, noaa has erratic dates but
% uhslc has regular dates with nan for missing data
    %missing data in 15min could have hourly spacing, but would still
    %indicate missing. The clue is that the spacing changes (by more than 
    %say 3 minutes timing error) regularly (say more than 10 times)
    if sum(diff(diff(dates))*24*60>3)>10;
        erratic_timing=1;
    else
        erratic_timing=0;
    end

%% Find skew surges per tidal cycle (c.wilson)

iht=0; %number of high tides

NhtEst=floor(Nt/12); %Generous estimate of number of high tides

%Preallocate, but the length might be a bit smaller
peakObs=nan(NhtEst,1); %index of ht in this timeseries
maxSealevel=nan(NhtEst,1); % sealevel at this ht
datesmaxTide=nan(NhtEst,1); %date/time of predicted HW
datesskews=nan(NhtEst,1); %tskew, but in datenum format
maxTide=nan(NhtEst,1); %predicted high tide within 6.25 hours of this measured peak %avoid risk of getting onto neighbouring tide
skews=nan(NhtEst,1); %measured ht at peak - predicted ht nearby (may be different time)

range6=round(6.25./diffdateshr); %how many records are in 6 hours
range6s=max(range6(1)+1,7); %start at least 6.25 hours & 6 records after beginning of series
range6e=max(range6(end)+1,7); %end at least 6.25 hours & 6 records before beginning of series

for n=range6s:(Nt-range6e); %index of timeseries
    
    %The nonans criteria works on the UK data, which contains nans, but the
    %Irish data - at least at Castletownbere - has no nans
    
    if diffdateshr(n)>0.9&&diffdateshr(n)<1.1 %hourly data criteria
        SLmaxcriteria=(dates(n+4)-dates(n-4))<9/24 ...
            &&slnonans(n-4)&&slnonans(n-3)&&slnonans(n-2)&&slnonans(n-1)...
            &&slnonans(n)...
            &&slnonans(n+1)&&slnonans(n+2)&&slnonans(n+3)&&slnonans(n+4)...
            &&(sealevel(n-1)< sealevel(n))&&(sealevel(n-2)< sealevel(n))...
            &&(sealevel(n-3)< sealevel(n))&&(sealevel(n-4)< sealevel(n))...
            &&(sealevel(n)>= sealevel(n+1))&&(sealevel(n)>= sealevel(n+2))...
            &&(sealevel(n)>= sealevel(n+3))&&(sealevel(n)>= sealevel(n+4));
    elseif  diffdateshr(n)>0.2&&diffdateshr(n)<0.3 %15 min data criteria %this is much slower
        SLmaxcriteria=slnonans(n)... %check nan first!
            &&(dates(n+16)-dates(n-16))<9/24 ... %+-4 hour window
            &&all(slnonans(n-16:n+16))...
            &&all(sealevel(n)>=sealevel(n-16:n+16));
    elseif  diffdateshr(n)>0.08&&diffdateshr(n)<0.12 %6min data criteria %this is much slower
        if n>(Nt-40)
            SLmaxcriteria=0; 
        else
        SLmaxcriteria=slnonans(n)... %check nan first!
            &&(dates(n+40)-dates(n-40))<9/24 ...
            &&all(slnonans(n-40:n+40))...
            &&all(sealevel(n)>=sealevel(n-40:n+40));
        end
    else %This is pretty strict, doesn't allow eg 10 minute data!
        SLmaxcriteria=0;
    end
    
    if     SLmaxcriteria
        %The problem here is that for 15-min data we're only looking 45 min
        %either side, so finding too many local maxima. It would be better
        %to have a stricter criteria for 15-min data. (diffdateshr~0.25)
        %Double low tides cause problems
        
        iht=iht+1; %Increment the maximum counter
        peakObs(iht)=n;
        maxSealevel(iht)=sealevel(n);
        datesskews(iht)=dates(n);
        
        %range8=round(8/diffdateshr(n));
        %maxTide(iht)=max(predictedtide(n-range8:min(n+range8,Nt))); %predicted high tide within 8 hours of this measured peak
        
        %This is slower but more reliable. %
        %idateswithin6=abs(dates-dates(n))<6.25/24;
        %ndateswithin6=find(idateswithin6); %use the index not just the logical
        if ~erratic_timing
                %Quicker (doesn't work in Ireland or USA UHSLC, no nans used)
                ndateswithin6=(n-range6(n)):(n+range6(n));%abs(dates-dates(n))<6.25/24;
                if diffdateshr(ndateswithin6(1))~=diffdateshr(ndateswithin6(end))
                    %deals with rare case of change in time between 1hr and 15 min
                    ndateswithin6=find(abs(dates-dates(n))<6.25/24);
                end
        else
                ndateswithin6=find(abs(dates-dates(n))<6.25/24);
        end
        
        %[maxTide(iht),iidatesHW]=max(predictedtide(idateswithin6)); %predicted high tide within 8 hours of this measured peak
        %Instead of looking for the maximum, look for the point at which
        %the tide turns - this avoids the problem illustrated in BewareExample6.25hours.fig
        %where the second HW is bigger than the first
        %this line can pick up a minimum if there are 3 points the same
        %iidatesHW=find([diff(predictedtide(idateswithin6));1]<=0&[-1;diff(predictedtide(idateswithin6))]>=0,1,'first');
        [maxTide(iht),iidatesHW]=maxpeak(predictedtide(ndateswithin6));
        %maxTide(iht)=predictedtide(idateswithin6(iidatesHW));
        datesmaxTide(iht)=dates(ndateswithin6(iidatesHW));
        skews(iht)=maxSealevel(iht)-maxTide(iht);
        
        %If maxima are within 10 hours and the second is larger then
        %replace the previous peak
        if iht>1&&((datesskews(iht)-datesskews(iht-1)) < 10/24)&&(maxSealevel(iht)>maxSealevel(iht-1))
            peakObs(iht-1)= peakObs(iht);
            maxSealevel(iht-1)= maxSealevel(iht);
            datesskews(iht-1)=datesskews(iht);
            datesmaxTide(iht-1)=datesmaxTide(iht);
            maxTide(iht-1)= maxTide(iht);
            skews(iht-1)=skews(iht);
            iht=iht-1;
        end
        %If maxima are within 10 hours and the second is smaller then just ignore the peak
        if iht>1&&((datesskews(iht)-datesskews(iht-1)) < 10/24)&&(maxSealevel(iht)<maxSealevel(iht-1))
            iht=iht-1;
        end
        
    end
    
end
%Tidy up surplus preallocation:
peakObs(iht+1:NhtEst)=[]; %index
maxSealevel(iht+1:NhtEst)=[];
maxTide(iht+1:NhtEst)=[];
datesskews(iht+1:NhtEst)=[];
datesmaxTide(iht+1:NhtEst)=[];
skews(iht+1:NhtEst)=[];

Nht=iht; %final number of high tides found




end
function [m,i]=maxpeak(x)
%return maximum from the data x
%Should return a single maximum that must be bigger than its immediate neighbours
i=find([diff(x);1]<=0&[-1;diff(x)]>=0);
%find points that are bigger than neighbours on either side
if length(i)==1 %usual
    m=x(i);
elseif isempty(i) %May happen rarely if a false peak in sea level at mid-tide.
    m=nan;
    i=1;
elseif length(i)>1 %May happen if there's a flat section in x or a flat minimum as well as a max
    [m,ii]=max(x(i));
    i=i(ii);
else
    error('This really ought not happen')
end
end