function [heightsout,datesout]=hourly2daily(heightsin,datesin,flag,filtertype)

%HOURLY2DAILY take hourly tidal data and return as daily means with dates set at noon
%
%Default is to use the Doodson filter, to remove tidal power
%Check this !
%
%Joanne Williams, joll@noc.ac.uk, 2018

%Ensure input(s) are column vectors, if not output is in the same format
if isrow(heightsin)
    returnrowheights=1; 
else 
    returnrowheights=0;
end

if isrow(datesin)
    returnrowdates=1; 
else 
    returnrowdates=0;
end

datesin=datesin(:);
heightsin=heightsin(:);

if ~isempty(flag) %Not implemented!
warning('flagged data not handled in filter')
end    


%Check that input data is hourly with no gaps (gaps should have been filled
%already)
if any(abs(diff(datesin)*24-1)>0.0001)
    error('Input for Doodson filter should be regular hourly data')
end

switch lower(filtertype)
    case 'doodson'
        %Define Doodson filter:
        F0=0;
        Fm=[2,1,1,2,0,1,1,0,2,0,1,1,0,1,0,0,1,0,1]./30;
        wts=[Fm(end:-1:1),F0,Fm];
        
        % !
        % ! NOW X0 FILTER THE HOURLY SEA LEVEL VALUES TO GET DAILY MEANS.
        % ! WORK ON THE VALUE WITHIN 1/2 HOUR OF MIDDAY.
        % !!
        [heightsout,datesout]=filthourlytodaily(heightsin,datesin,flag,wts);
        
    case 'demerliac'
               %Demerliac filter from SONEL.org after Bessero G. (1985). Marées. SHOM, Fascicule 2, chap. IX à XV.
        wts=[1, 3, 8, 15, 21, 32, 45, 55, 72, 91, 105, 128, 153, ...
 			171, 200, 231, 253, 288, 325, 351, 392, 435, 465, 512, ...
 				558, 586, 624, 658, 678, 704, 726, 738, 752, 762, 766,...
 			768, 766, 762, 752, 738, 726, 704, 678, 658, 624, 586, ...
 			558, 512, 465, 435, 392, 351, 325, 288, 253, 231, 200,...
 			171, 153, 128, 105, 91, 72, 55, 45, 32, 21, 15, 8, 3, 1 ]./24576;
        
          [heightsout,datesout]=filthourlytodaily(heightsin,datesin,flag,wts);
    case 'nofilter'
         %provide an option of just doing an arithmetic mean 
         %dates returned at noon and nans propogate
        [heightsout,datesout]=dailymeans(heightsin,datesin,0.5,0);         
        
    case 'nofilter_usenanmean'
         %provide an option of just doing an arithmetic mean 
         %dates returned at noon and nans propogate
        [heightsout,datesout]=dailymeans(heightsin,datesin,0.5,1);   
        
    otherwise
        error('filter not defined')
end



if returnrowheights
     heightsout=heightsout(:)';
end

if returnrowdates
    datesout=datesout(:)';
end

end

%Subfunction
function [heightsout,datesout]=filthourlytodaily(heightsin,datesin,flag,wts)

Nt=length(datesin);

%filter is [wts,wts(end-1:-1:1)]
Nfilt=length(wts);
Nhalf=(Nfilt-1)/2; %exclude central value

%List of days (00:00) that fall in the date range.
days=unique(floor(datesin));

%Indices of datesin nearest to each noon.
itnoon=round(interp1(datesin,1:Nt,days+0.5,'nearest'));

%if a noon in the date range 

Nday=length(itnoon);

datesout=days;
heightsout=nan(Nday,1);
for iday=1:Nday
    it=itnoon(iday); %this is the central value for the filter
    
    if isnan(it)||it<=Nhalf||(Nt-Nhalf+1)<=it %nan test is to eliminate problems from interpolation step
        datesout(iday)=nan;
        heightsout(iday)=nan;
    else
        %multiply weights by heights
        heightsout(iday)=sum(wts(:).*heightsin(it+[-Nhalf:Nhalf]));
    end
    
end

%Get rid of anything with a nan *DATE*
heightsout(isnan(datesout))=[];
datesout(isnan(datesout))=[];

end