%SAVEFIGURE  save figure as jpg, fig, and line data in binary format
%
% SAVEFIGURE(fighand,filestr)
% SAVEFIGURE(fighand,filestr,plothand,filetype,sqfig)
%
% Recommended use: 
%
% SAVEFIGURE(gcf,filestr,[],'png300pdf')
% 
% Or for figures that are mostly line drawings: 
% SAVEFIGURE(gcf,filestr,[],'epspdf')
% But the latter doesn't save a format which can be used in powerpoint
%
% If plothand is not specified, binary data is only saved for the first
% plotted line of the current axis
%
% Joanne Williams joll@pol.ac.uk August 2009
%

% With the renderer set to 'painters', WYSIWYG. But eveything is much
% slower!
% Renderer set to 'zbuffer' or 'Opengl' can cause the fonts to be redone -
% and even tick-marks moved during the export.
% This happens at the stage of "saveas" to eps2c.

function savefigure(fighand,filestr,plothand,filetype,sqfig)

set(fighand,'Filename',[filestr,'.fig']);

if ~strncmp(filetype,'nofig',5)
saveas(fighand,[filestr,'.fig'],'fig')
else
    disp(['Will be saving format: ' filetype])
end
drawnow

if nargin>=5&&strcmp(sqfig,'sq')
    %when you've got square axes, trim whitespace on paper & screen
    ap=get(gca,'pos');
    pos=get(gcf,'Pos');
    set(gcf,'Pos',[pos(1),pos(2),pos(4)*ap(4)/ap(3),pos(4)])
    pp=get(gcf,'Paperp');
    set(gcf,'paperPos',[pp(1),pp(2),pp(4)*ap(4)/ap(3),pp(4)])
    %axis square
end

if nargin>=4&&strcmp(filetype,'eps')
    saveas(fighand,[filestr,'.eps'],'eps2c')
elseif nargin>=4&&strcmp(filetype,'epspdf')
    %Go to pdf via vector format
    %May be very large files for maps
    set(fighand, 'PaperPositionMode', 'auto');
    set(gca,'xtickmode','manual','ytickmode','manual')
    saveas(fighand,[filestr,'.eps'],'eps2c')
    %eps2pdf([filestr,'.eps'],[filestr, '.pdf'])
    
    system(['epstopdf ' filestr '.eps '])
    %This is the CTAN utility.
elseif nargin>=4&&strcmp(filetype,'pngpdf')
    %Go to pdf via bitmap format. Handles maps well.
    %May be lossy for text?
    set(fighand, 'PaperPositionMode', 'auto');
    set(gca,'xtickmode','manual','ytickmode','manual')
    saveas(fighand,[filestr,'.png'],'png')
    
    system(['convert ' filestr '.png ' filestr '.pdf'])
    %Note that "convert" is the commandline form of ImageMagick. It is not
    %suitable for vector images (ie eps). It will be OK for png.
    %Really the png should have a resolution associated with it, suitable
    %to the image and/or the final production quality.
    %For a figure that fills 1/2 an A4 page this would be ~150dpi at the
    %Matlab defaults. This may be in the "print" settings.

elseif nargin>=4&&(strcmp(filetype,'png300')||strcmp(filetype,'nofigpng300'))
    %Go to pdf via bitmap format. Handles maps well.
    %May be lossy for text?
    set(fighand, 'PaperPositionMode', 'auto');
    set(gca,'xtickmode','manual','ytickmode','manual')
    print(fighand,[filestr,'.png'],'-dpng','-r300') %300dpi
    
elseif nargin>=4&&(strcmp(filetype,'png300pdf')||strcmp(filetype,'nofigpng300pdf'))
    %Go to pdf via bitmap format. Handles maps well.
    %May be lossy for text?
    set(fighand, 'PaperPositionMode', 'auto');
    set(gca,'xtickmode','manual','ytickmode','manual')
    print(fighand,[filestr,'.png'],'-dpng','-r300') %300dpi
    
    system(['convert ' filestr '.png ' filestr '.pdf'])
    %Note that "convert" is the commandline form of ImageMagick. It is not
    %suitable for vector images (ie eps). It will be OK for png.
    
elseif nargin>=4&&(strcmp(filetype,'png600pdf')||strcmp(filetype,'nofigpng600pdf'))
        %This may be higher resolution than most printers can manage - and
        %may be expensive, nearly as high resolution as the eps file.
    %Go to pdf via bitmap format. Handles maps well.
    %May be lossy for text?
    set(fighand, 'PaperPositionMode', 'auto');
    set(gca,'xtickmode','manual','ytickmode','manual')
    print(fighand,[filestr,'.png'],'-dpng','-r600') %300dpi
    
    system(['convert ' filestr '.png ' filestr '.pdf'])
    %Note that "convert" is the commandline form of ImageMagick. It is not
    %suitable for vector images (ie eps). It will be OK for png.
    
    
    
else
    %Go to jpg. Lossy compression.
    saveas(fighand,[filestr,'.jpg'],'jpg')
    
end

if nargin<3&&~isempty(get(gca,'ch'))
    ch=get(gca,'ch');
    plothand=ch(end);
elseif nargin<3
    plothand=[];
end

if length(plothand)==1
    %Check that the plothand refers to line data
    if strcmp(get(plothand,'Type'),'line')
        
        xdat=get(plothand,'xdata');
        fid=fopen([filestr, '.xdat'],'w');
        fwrite(fid,xdat,'double');
        fclose(fid);
        
        ydat=get(plothand,'ydata');
        fid=fopen([filestr, '.ydat'],'w');
        fwrite(fid,ydat,'double');
        fclose(fid);
    end
    
else
    for i=1:length(plothand)
        %Check that the plothand refers to line data
        if strcmp(get(plothand(i),'Type'),'line')
            
            xdat=get(plothand(i),'xdata');
            fid=fopen([filestr,'.' num2str(i) '.xdat'],'w');
            fwrite(fid,xdat,'double');
            fclose(fid);
            
            ydat=get(plothand(i),'ydata');
            fid=fopen([filestr, '.' num2str(i) '.ydat'],'w');
            fwrite(fid,ydat,'double');
            fclose(fid);
        end
    end
    
end

disp([filestr,'.fig'])
%disp([filestr,'.jpg'])


%This is a good opportunity to check whether any files have accidentally
%been left open
if ~isempty(fopen('all'))
    warning('savefigure:OpenFiles','Files have been left open somewhere, check fclose operations')
end
