function tidegaugeQC(sitecode,dateS,dateE,varargin)

% TIDEGAUGEQC Apply Quality Control
%
% Usage:
% TIDEGAUGEQC(SITECODE,DATES,DATEE,VARARGIN)
%   SITECODE is a tide-gauge code from the IOC website, eg 'pric'
%   DATES    start date for data to process
%   DATEE    end date for data to process
%
% Pairs of arguments and defaults
% 'rootdatadir'   :'./'    - path to find work in, eg '/work/dataQC/data/' %'C:\Users\fred\dataQC\data\';
% 'sitename'      :SITECODE - Full name of site, eg 'Prickly Bay'
% 'excludesensors':{}       - Exclude these sensors,
%                              in addition to {'bat','sw1','sw2'}.         
%                              All other sensors found are processed. 
% 'usesensors',   :{}       - Process only these sensors, eg {'prs','rad'}. 
%                              This option overrides 'excludesensors'
% 'relativetochan1': 
% 'maxyearswarning':1        
% 'dataunits'      :'m'     - Provision for future units option (not implemented)
% 'outofrangevalues':{}      - Provision for flagging of out-of-range values
%                              eg {[a1,a2],[b1,b2],[c1,c2]}
%
% 'datumcorrection':[]      -
% 'FlagChannelDiffGreaterThan': 0.1 - Default difference between channels (hourly, after initial QC) to flag,
%                            (as well as flagging based on median difference).
%                            Set to FlagChannelDiffGreaterThan=inf; to ignore this option.
% 'StitchChannel'  :1       - Return a single stitched channel
% 'chanrank'       :[]      - Provision for ranking channels for stitching purposes only
% 'showspikefigs'  :'off'   - Set to 'verbose' to Generate spike figures during flagging
% 'usePughfilters' :0       - Select final averaging of non-tidal residuals 
%                           - Set to 0 to use simple hourly averages 
%                           - Set to 1 to use Pugh/Vassie filters - this will make the data more gappy
%   
% 
% =====Dependencies======:
% Matlab libraries, including statistics toolbox.
% PARSE_PV_PAIRS (from Matlab Central, John d'Errico)
% 
% Functions to read from the IOC website, and reload locally:
% readIOC.m	
% readIOCmeta.m
% loadlocalIOC.m
%
% Tide processing library 
% NOCtidefit_ctlwrapper.m 
% NOCtidepred_wrapper.m 
% NOCtidefit.m (from Simon Williams, NOC)
% NOCtidepred.m (from Simon Williams, NOC)
% phamp0fast.m (from Simon Williams, NOC)
% ufsetfast.m (from Simon Williams, NOC)
% vsetfast.m (from Simon Williams, NOC)
% longfindfast.m (from Simon Williams, NOC)	
% readctl.m (from Simon Williams, NOC)
% 
% Quality control and tide-related functions
% spike_detect_spline.m
% stuck_detect_test.m
% stuck_osc_detect_test.m
% getskewsurge.m
% channelsimilar.m
% channelbetter.m
%
% Averaging functions:
% daily2mon.m
% dailymeans.m
% hourly2daily.m
% hourlymeans.m	
% Nminmeans.m
% filltoregular.m
% filtto60min.m
%
% Plotting tools:
% patchbackgroundx.m
% savefigure.m	    
% plotwithgaps.m
%
% Other tools:
% modnear.m
% polyfitNoTests.m
% rounddatesec.m  
%
% ======Called by:=======
% runQCfundemo.m
%
%
% ========================
%
% For full explanation and examples see report on CME programme 
% Williams, Joanne; Matthews, Andrew; Jevrejeva, Svetlana. 2019 
% Development of an automatic tide gauge processing system. 
% Southampton, National Oceanography Centre, 26pp.
% (National Oceanography Centre Research and Consultancy Report, 64)
%
% This is available via www.psmsl.org/cme
%
% Joanne Williams joll@noc.ac.uk March 2019 

% Preferred channel
%There may be information from the operator about the preferred channel at
%some point. For example, start on the radar sensor but switch to the
%pressure gauge at a certain data when a known failure occurred, and back
%after repairs.
%For now, assume this information is not available as we don't know what
%format it might be in.

%FUTURE DEVELOPMENT:
%To call the QC routine with a timeseries instead of an IOC sitecode
%
%We need a way to abort safely if the data is *all* flagged


%% Set Defaults and parameters

params.rootdatadir='';%/work/joll/dataQC/data/'; %'C:\Users\dazzler\Documents\CMEPdataQC\forworkshop\data\';
params.sitename=sitecode; %Ideally this is provided as eg "Prickly Bay" instead of "pric"
params.relativetochan1=[];
params.maxyearswarning=1; %
params.usesensors={};
params.excludesensors={};%These sensors are excluded in addition to {'bat','sw1','sw2'}
params.dataunits='m'; % Provision for future units option (not implemented)
params.outofrangevalues={}; %eg: {[a1,a2],[b1,b2],[c1,c2]} Provision for flagging of out-of-range values
params.datumcorrection=[];
params.FlagChannelDiffGreaterThan=0.1; 
%Default difference between channels (hourly, after initial QC) to flag,
%(as well as flagging based on median difference).
%Set to FlagChannelDiffGreaterThan=inf; to ignore this option.

params.StitchChannel=1; %Return a single stitched channel
params.chanrank=[]; %Provision for ranking channels for stitching purposes only
params.showspikefigs='off'; %Set to 'verbose' to Generate spike figures during flagging
params.usePughfilters=0;

%Parameters that are set in varargin:
params=parse_pv_pairs(params,varargin);

%% Allow (reluctantly) read-write in local directory 

rootdatadir=params.rootdatadir;
if isempty(rootdatadir)
    warning('No absolute data path provided?')
    disp('Seeking and Writing data from current directory.')
    disp(pwd);
    rootdatadir='./';
end

%Check for trailing backslash in directory name
if ~(strcmp(rootdatadir(end),'/')||strcmp(rootdatadir(end),'\'))
    rootdatadir=[rootdatadir,'/'];
end

%% Data and Figure Directories

datadir=[rootdatadir  sitecode '/'];

basefile=[sitecode, datestr(dateS,'_yyyymmdd'), datestr(dateE,'_yyyymmdd')];


%% Check date range and warn of potential memory problems if on a PC

maxyearswarning=params.maxyearswarning;
if ispc && ((dateE-dateS)>(maxyearswarning*365+10)) %allow 10 days over max years before warning of memory issues
    warning('You may have memory problems analysing long records')
    reply = input('Continue? Y/N: ','s');
    
    if ~strcmpi(reply,'Y')
        return
    end
    
end

%% Access remote data. Download from IOC site.

%Access metadata (eg latitude). Store locally for reuse.
sitemetadata=readIOCmeta(datadir,sitecode); %!Put an 'auto' option here in case we want to force a fresh download

readIOC(sitecode,dateS,dateE,datadir,'auto');
%If readIOC.m finds you've done this site and dates before,
%you will have the option not to download again. 'auto' assumes this. 

%% Select sensors for consideration 

% All sensors at the site
Nsens=length(sitemetadata.sensors);
for isens=1:Nsens
    sensornames{isens}=sitemetadata.sensors(isens).sensor; %#ok<AGROW>
end

% Identify and discard sensors which aren't the main data (eg batteries, switches)
standardexcludelist={'bat','sw1','sw2'};
excludesensornames=union(params.excludesensors,standardexcludelist);%;

% Or explicit list of sensors 
usesensornames=params.usesensors;

usechan=zeros(1,Nsens); %Ensure a row vector or it breaks the lazy for-loop later.
if ~isempty(usesensornames) %If usesensornames specified, use this list:
    for isens=1:Nsens
        usechan(isens)=any(strcmp(sensornames{isens},usesensornames));
    end
else %use all sensors that are not in the excluded list
    for isens=1:Nsens
        usechan(isens)=~any(strcmp(sensornames{isens},excludesensornames));
    end
end

% Also exclude any sensor that is not in metres 
% (FUTURE - allow other heights and convert)
for isens=1:Nsens
    if ~strcmpi(sitemetadata.sensors(isens).units,'m')
        usechan(isens)=0;
    end
end

% Then chanvec is a row vector of the selected channels
% chanvec is only meaningful with *THIS* list of sensors. 
chanvec=find(usechan);

save([datadir,basefile,'_sensornames.mat'],'sensornames','chanvec');

if isempty(chanvec)
    error('No channels specified?')
end

%% Other preparation. 

% Provision for future units option (convert to m if necessary)
metadataunits=params.dataunits;%'m';
if ~strcmp(metadataunits,'m')
    error('This code expects data in m, metres');
    %Need to implement conversion of data if this is set
end

% Provision for future flagging of out-of-range values.
if ~isempty(params.outofrangevalues)
    metarangecell=params.outofrangevalues;
    if length(metarangecell)~=Nsens
       error('Mismatch between sensors and sensor out-of-range values provided')
    end
else    
    warning('No sensor out-of-range values provided, setting to 1e9')
    for ichan=chanvec
        metarangecell{ichan}=[-1e9,+1e9]; %#ok<AGROW> %Set default to huge number in case used for eg ocean bp in pascal
    end
end

% Datum shifts
datumcorrection=params.datumcorrection;
if ~isempty(datumcorrection)
    if length(datumcorrection)~=Nsens
       error('Mismatch between sensors and datum correction provided')
    end
    warning('Sorry, at the moment no datum correction is implemented in the code!')
end

% Relative datums of the channels
%   If specified, this is used instead of medians to compare channels for QC
%   No absolute datums are specified.
%   Channel numbers should match the sensornames list specified above.
%   It is particularly important at Calliaqua, where one sensor has a severe
%   drift compared to the others, so the median of the full series is not a
%   useful measure.
%   Set to empty if not known or required.
relativetochan1=params.relativetochan1;

%% Primary QC for each sensor individually (before any tidal analysis)
%Preallocate output
predcell=cell(1,Nsens);

for ichan=chanvec
    sensstr=sensornames{ichan}; %eg prs
    [dates,heights,sensorbasefile]=loadlocalIOC(datadir,sitecode,sensstr,...
        dateS,dateE);
    sensorbasefilecell{ichan}=sensorbasefile; %#ok<AGROW>
    
    %% Strange Character Detection (not implemented)
    
    %% Ensure dates are in UTC (not implemented)
    
    %% Incorrect units (not implemented)
    % what would we do if the units suddenly changed? Or were wrong right
    % through?
    
    %% Datum shift (not fully implemented - manual corrections applied for certain gauges)
    % Original datums should be preserved for PSMSL.
    % The tidal analysis is to some degree robust against datum shift, it should be
    % preserved, but severe datum shifts do disrupt the constituents.
    % Flag datum shift and define an estimated datum series (Heaviside) to pass
    % to tidal analysis
    
    %% Flagging out of range values based on metadata for each station
    isoutofrange=heights<metarangecell{ichan}(1)|metarangecell{ichan}(2)<heights;  %1=BAD 0=GOOD
    outofrangeflags=ones(size(isoutofrange));  %1=GOOD 4=BAD
    outofrangeflags(isoutofrange)=4;

    %% Estimate some parameters about the data.
    
    % Find the precision (number d.p) of the data in the channel (may be needed for settings)
    precisionest{ichan}=min(diff(uniquetol(heights,1e-10))); %#ok<AGROW>
    if isempty(precisionest{ichan})
        precisionest{ichan}=1e-10; %#ok<AGROW>
    end %Ensure it doesn't set to 0 for completely flat return
    %eg 0.001 for 1mm
    
    % Estimate the usual time interval of data in the channel (may be needed for settings)
    %timedelta{ichan}=median(diff(dates))*24*60; %#ok<AGROW> %time interval in minutes
    timedelta{ichan}=min(diff(dates))*24*60; %#ok<AGROW> %time interval in minutes,
    if timedelta{ichan}<1
        sprintf('Sampling interval %f minutes?', timedelta{ichan});
        warning('Assuming sampling interval of 1 minute')
        timedelta{ichan}=1; %#ok<AGROW> %time interval in minutes,
    end
    sampleperhour{ichan}=(60/timedelta{ichan}); %#ok<AGROW>
    
    % Estimate the usual standard deviation of the data over 60 points
    typicalstd{ichan}=median(movstd(heights,60)); %#ok<AGROW>
    
    %% Stability tests (partially implemented)
    % "Stability" test: flagging values when there is no change in the
    % magnitude of sea level after a number of time steps.
    % Partially implemented: fixed at flagging 5 in a row
    stuckflags=stuck_detect_test(heights,timedelta{ichan}); %1=GOOD, 4=BAD.

    % Stuck oscillating flag (not in Gloss)
    % flag sections of data when the instrument jumps frequently to a fixed value
    stuckoscflags=stuck_osc_detect_test(heights,precisionest{ichan}); %1=GOOD, 4=BAD.
    
    % Combine the results of these tests:  %1=GOOD, 4=BAD.
    stuckflags(stuckoscflags==4)=4;
    stuckflags(outofrangeflags==4)=4;
        
    save([datadir sensorbasefile '_stuckflags.mat'],'stuckflags');
    
    %% Spike detection (flag 4=bad used for consistency with Gloss)
    %
    % The spike detection algorithm allows for irregular data.
    % Note that some spikes could be genuine records for high-frequency waves.
    % But they should definitely be discarded for tidal analysis
    % Tests on tsunami data from Chile show that most genuine events are
    % retained.
    %
    % 31 Jan 2019 : This has been moved to be after the "Stuck" test as
    % nearby stuck values can cause difficulties with spike detection.
    % 1st Feb: The spike detection algorithm has been altered to use
    % medians, and to be repeated to improve performance. it is now much more robust and
    %
    %Note that in Gloss manual(?)
    %flag==1 is good, flag==4 is BAD, flag==9 is MISSING (so could be used instead of Nan when filling data)
    %flag==5 Value changed (could use this for interpolated data?)
    
    try % if spike detection already done, reload to save time during testing.
        %    error('Force spike detection!')
        if ~exist([datadir,sensorbasefile '_spikeflags.mat'],'file')
            error([datadir,sensorbasefile '_spikeflags.mat' ' : File does not exist'])
        end
        load([datadir sensorbasefile '_spikeflags.mat'],'spikeflags');
    catch
        
        %Temporarily replace stuck data with nan to avoid it interfering
        %with spline fitting, without changing the length of the output
        heightstemporary=heights;
        heightstemporary(stuckflags==4)=nan;
        
        splineparam.Nwin=60;
        splineparam.minspike=3*typicalstd{ichan};
        
        spikeflags1st=spike_detect_spline(heightstemporary,dates,splineparam,params.showspikefigs);
        
        heightstemporary(spikeflags1st==4)=nan;
        spikeflags2nd=spike_detect_spline(heightstemporary,dates,splineparam,params.showspikefigs);
        
        
        spikeflags=spikeflags1st;
        spikeflags(spikeflags2nd==4)=4;
        
        save([datadir sensorbasefile '_spikeflags.mat'],'spikeflags');
        %DemoPlotsPric2.m
        
    end
    
    % Spike detection - if lots of spikes have the same value, this might be
    % a deliberate fill value. In this case, assume all others at this value
    % are probably spikes? Dangerous - as it might remove eg all instances
    % where a radar gauge is overtopped.
    
    % Note that Phil has provided a link to some code meant for spotting
    % tsunamis that might be useful here.
    
    
    %% Check for nans
    heightsnan=isnan(heights);
    
    %% Filtering to produce hourly data
    %
    % This first hourly filter is used for further flagging before tidal analysis
    %
    % At this stage don't worry about the Vassie filters
    % Just do a simple hourly filter, as we just need a rough tide
    % and comparison of the channels, and we don't want nans to propogate.
    %
    % Note that unlike filtto60min, the hourlymeans function doesn't fill
    % gaps in the date series. Therefore the dates are still not continuous.
    % This has implications for the behaviour of the channel comparison
    % as there will never be nans in the heights. We can double check by
    % stripping nans on input
    % This should make it much easier than using filtto60min.
    %
    % "60" is used here in variable names to distinguish from "hourly"
    % later. (not very consistently - check!)
    
    [heights60,dates60]=hourlymeans(heights(spikeflags==1&stuckflags==1&~heightsnan),...
        dates(spikeflags==1&stuckflags==1&~heightsnan)+1/48,0,1);
    
    %% Export this channel
    datescell{ichan}=dates; %#ok<AGROW>
    heightscell{ichan}=heights; %#ok<AGROW>
    spikeflagscell{ichan}=spikeflags; %#ok<AGROW>
    stuckflagscell{ichan}=stuckflags; %#ok<AGROW>
    
    heights60cell{ichan}=heights60; %#ok<AGROW>
    dates60cell{ichan}=dates60; %#ok<AGROW>
    
    % Estimates of the central value to provide an offset between instruments
    % **NOT THE DATUM!**
    % Exclude anything already flagged
    
    meanheightscell{ichan}  =  mean(heights(spikeflags==1&stuckflags==1&~heightsnan)); %#ok<AGROW>
    medianheightscell{ichan}=median(heights(spikeflags==1&stuckflags==1&~heightsnan)); %#ok<AGROW>
    
    % In case of drift or occasional extremes, median probably represents a
    % more useful statistic for estimating instrument offset than mean.
    % However if there is a datum shift near the middle of the record, the median will
    % lie somewhere between the datums for the beginning and end. (the
    % distribution is bimodal)
    % In this case it is more useful to ensure the median is set for the same
    % period everywhere.
    % Eg (more diagnostics, comment in if you need to use them to set relativetochan1):
    %
    %     finalyearmed{ichan}=median(heights(spikeflags==1&stuckflag==1&~heightsnan&dates>=(dateE-365))); %#ok<AGROW>
    %     firstyearmed{ichan}=median(heights(spikeflags==1&stuckflag==1&~heightsnan&dates<=(dateS+365))); %#ok<AGROW>
    %     mostcommonmed{ichan}=mostcommonmedian(heights(spikeflags==1&stuckflag==1&~heightsnan),...
    %         round(sampleperhour{ichan}*24*30));
    
end

%% Relative datum corrections for each channel
% No absolute datum corrections are yet applied.
%
% Correction required on each channel to bring them to the same datum (note
% this is only a relative correction, not to any external datum).

if ~isempty(relativetochan1)
    userelativechan=1;
    for ichan=chanvec
        datumcor{ichan}=relativetochan1(ichan); %#ok<AGROW>
    end
else
    userelativechan=0;
    warning('No relative channel datum specified, calculating from median unflagged.')
    for ichan=chanvec
        datumcor{ichan}=medianheightscell{ichan}; %#ok<AGROW>
    end
end

%% Look for massive deviations between sensors.
%This might pick up a sensor that has failed temporarily or in a surprising way
% chanSIM=1(true) for SIMILAR to another,  0(false) for DIFFERENT
[chan60flagSIMcell,chanflagSIMcell]=channelsimilar(chanvec,...
    dates60cell,heights60cell,...
    datescell,heightscell,...
    datumcor,params.FlagChannelDiffGreaterThan);

%% Initial tidal analysis - only on unflagged spikeless data where both channels agree!

%There is a potential problem with the method of only doing tidal analysis
%on consistent channels. If there's one channel that continues to run although
%it's complete rubbish (it happens at St Lucia, though on that gauge there's
% 2 others, and it does get switched off after a while) then that channel
%should be ditched early, and not allowed to prevent the other channel(s)
%from providing enough data for tidal analysis.
%
%If all of a channel is flagged as dissimilar (say there's several datum
%shifts) we need to pick *something* to use for the initial tidal analysis.
%Otherwise it might be a really short random section that will give very
%bad results. It may be better a better approach to only have one channel
%for initial tidal analysis.
%
%To deal with these problems, if there is less than 20 days contiguous data
%or 100 days gappy data, so NOCtidefit_ctlwrapper refuses to attempt a
%tidal fit, then instead all the unflagged data for that channel is passed
%for initial analysis.

for ichan=chanvec
    % Lazy reference to each channel
    sensstr=sensornames{ichan}; %eg prs
    dates=datescell{ichan}(:);
    dates60=dates60cell{ichan}(:);
    heights=heightscell{ichan};
    heights60=heights60cell{ichan};
    spikeflags=spikeflagscell{ichan};
    stuckflags=stuckflagscell{ichan};
    chanflagSIM=chanflagSIMcell{ichan}; % =1(true) for SIMILAR,  0(false) for DIFFERENT
    
    % Before tidal analysis we should discard spikes and stuck data.
    % We could also consider removing estimated datum shifts.
    % But it is not necessary to perform interpolation of gaps (which is
    % done by PdE group), or to have data on regular intervals.
    
    % Also before tidal analysis check for extremes (high or low)
    % Some of these may be genuine surge/tsunami, but we should omit them
    % *during the initial tidal predictions*.
    % It's worth it to mop up bad data that's somehow been missed by the
    % other tests, for the sake of an approximate analysis.
    
    % extremeflags=1 for good, 3 for probably bad
    extremeflags=ones(size(heights));
    extremeflags(heights<quantile(heights(spikeflags==1&stuckflags==1&chanflagSIM==1),0.001)|...
        heights>quantile(heights(spikeflags==1&stuckflags==1&chanflagSIM==1),0.999))...
        =3;
    
    % Note on tidal analysis - a timing discrepancy of 5 minutes, which could easily arise
    % from bad interpolation of 15 min data - is enough to muck up surge
    % residual results due to phase errors!
    
    % Note that if these flags are passed directly to NOCtidefit, then
    % in that software 'flag' is 1 for GOOD data or 0 for BAD data
    % But NOCtidefit doesn't handle constituent selection well if there's lots of
    % data gaps.
    % So use the wrapper NOCtidefit_ctlwrapper to check data length and
    % select constituents
    [amp,phase,~,names,p] = NOCtidefit_ctlwrapper(...
        dates(  extremeflags==1&spikeflags==1&stuckflags==1&chanflagSIM==1),...
        heights(extremeflags==1&spikeflags==1&stuckflags==1&chanflagSIM==1));
    
    %If NOCtidefit cannot fit a tide, perhaps because the data's too gappy
    %and short, then have a another attempt at fitting an initial tide to
    %this channel, including where the channels are dissimilar.
    %It may be a bad tide, but could still be better than nothing or the
    %tide fitted to a very short stretch?
    if all(isnan(amp))
        [amp,phase,~,names,p] = NOCtidefit_ctlwrapper(...
            dates(  extremeflags==1&spikeflags==1&stuckflags==1),...
            heights(extremeflags==1&spikeflags==1&stuckflags==1));
    end
    
    constsfile=[datadir,sensorbasefilecell{ichan},'_constituents'];
    save([constsfile,'initialtides.mat'],'amp','phase','names');
    
    [pred] =  NOCtidepred_wrapper(dates,amp,phase,'constant','on');
    [predhourly] =  NOCtidepred_wrapper(dates60,amp,phase,'constant','on');
    
    nontidalresidhourly=heights60-predhourly;
    nontidalresid60cell{ichan}=nontidalresidhourly; %#ok<AGROW>
    
end


%% Now switch back on the channel that is "best" during disagreements
%Note that this *doesn't* return a single preferred channel - that would
%require good knowledge of the relative datum in order to stitch them
%together, and another decision about which one to drop if there were 2
%channels

chanflagprefcell=channelbetter(chanvec,dateS,dateE,dates60cell,nontidalresid60cell,chan60flagSIMcell,datescell,chanflagSIMcell);
save([datadir basefile '_chanpref.mat'],'chanflagprefcell'); % 1=GOOD 3= PROBABLY BAD

%% Save all QC flags together
for ichan=chanvec
    spikeflags=spikeflagscell{ichan};
    stuckflags=stuckflagscell{ichan};
    chanflagpref=chanflagprefcell{ichan};
    combinedflags=ones(size(spikeflags)); 
    combinedflags(spikeflags==4|stuckflags==4)=4; % 1=GOOD 4=BAD
    combinedflags(chanflagpref==3)=3; % 1=GOOD 4=BAD 3 = PROBABLY BAD
    combinedflagscell{ichan}=combinedflags; %#ok<AGROW,NASGU> % 1=GOOD 4=BAD 3 = PROBABLY BAD
end
save([datadir basefile '_combinedflags.mat'],'combinedflagscell'); % 1=GOOD 0=BAD

%% Redo tidal processing, using all the consistent information

for ichan=chanvec
    % Lazy reference to each channel
    sensstr=sensornames{ichan}; %eg prs
    dates=datescell{ichan}(:);
    heights=heightscell{ichan};
    spikeflags=spikeflagscell{ichan};
    stuckflags=stuckflagscell{ichan};
    chanflagpref=chanflagprefcell{ichan};
    
    %Bad times in Channels flagged in chanflagprefcell are set to NaN for this
    heights(chanflagpref~=1)=nan;
    
    %Also before tidal analysis check for extremes (high or low)
    %Some of these may be genuine surge/tsunami, but we should omit them
    %during the tidal predictions.
    %It's worth it to mop up bad data that's somehow been missed by the
    %other tests
    
    %extremeflags 1= good, 3 = PROBABLY Bad
    extremeflags=ones(size(heights));
    extremeflags(heights<quantile(heights(spikeflags==1&stuckflags==1&chanflagpref==1),0.001)|...
        heights>quantile(heights(spikeflags==1&stuckflags==1&chanflagpref==1),0.999))...
        =3;
    
    % Use the wrapper NOCtidefit_ctlwrapper to check data length and
    % select constituents
    [amp,phase,~,names,p] = NOCtidefit_ctlwrapper(...
          dates(extremeflags==1&spikeflags==1&stuckflags==1&chanflagpref==1),...
        heights(extremeflags==1&spikeflags==1&stuckflags==1&chanflagpref==1));
    
    constsfile=[datadir,sensorbasefilecell{ichan},'_constituents'];
    save([constsfile,'tides.mat'],'amp','phase','names');
    savetideconsttotxt(constsfile,amp,phase,names,p)% write to format compatible with Jeff, for test
    
    [pred] =  NOCtidepred_wrapper(dates,amp,phase,'constant','on');
    
    nontidalresid=heights-pred;
    
    %% Filtering to produce hourly data, and/or monthly means
    
    % The Pugh/Vassie filters require regular data.
    % Because we have non-tidal residuals now we should be able to fill short
    % gaps with a linear fit. (Although not idea, this is standard, and
    % in some other systems is even done on the tidal data.)
    % It is not trivial to implement the filters for irregular data.
    % This is all done within filtto60min
    %
    % Another disadvantage of the Pugh/Vassie filters is that they use wide
    % windows and so substantially widen any data gaps.
    %
    % Since we've implemented the tidal processing *before* moving to hourly data -
    % which we can with NOCtidefit code - then we perhaps simply use hourly averages
    % rather then this system of interpolation and filtering... except that
    % there will still be tidal signal remaining in the event of a surge (due to
    % phase changes) and we don't want to alias that either.... Provide
    % options
    
    if params.usePughfilters
        % Filtering the non-tidal residuals
        [nontidalresidhourly,dateshourly]=filtto60min(nontidalresid(spikeflags==1&stuckflags==1),...
            dates(spikeflags==1&stuckflags==1));
    else
        % Alternative.
        [nontidalresidhourly,dateshourly]=hourlymeans(nontidalresid(spikeflags==1&stuckflags==1),...
            dates(spikeflags==1&stuckflags==1)+1/48,0,floor(sampleperhour{ichan}/2)); %accept an average of half data
        
        [nontidalresidhourly,dateshourly]=filltoregular(nontidalresidhourly,dateshourly,...
            (dateshourly(1):1/24:dateshourly(end))',nan);
    end
    
    %% RECONSTRUCT hourly data by adding tidal prediction back in
    %This ensures consistency of dates60 and (more importantly) that the
    %interpolation on heights60 has been done with the tide prediction removed
    %Note that where chanflagpref~=1, there were nans.
    [predhourly] =  NOCtidepred_wrapper(dateshourly,amp,phase,'constant','on');
    heightshourly=predhourly+nontidalresidhourly;
    
    %% Export this channel
    predcell{ichan}=pred;
    dateshourlycell{ichan}=dateshourly; %#ok<AGROW> %Saved at end of file
    predhourlycell{ichan}=predhourly;  %#ok<AGROW> %Saved at end of file
    nontidalresidhourlycell{ichan}=nontidalresidhourly;%#ok<AGROW> %Saved at end of file
    heightshourlycell{ichan}=heightshourly;%#ok<AGROW> %Saved at end of file
    
end

%% Recalculate Datum corrections for each channel if necessary
% No absolute datum corrections are yet applied.
% Correction required on each channel to bring them to the same datum (note
% this is only a relative correction, not to any external datum).

if userelativechan==1
    %No change
    datumcor_afterqc=datumcor;
else  %userelativechan=0;
    warning('No relative channel datum specified. Recalculating with best available data.')
    %At this stage the median should be consistent with the whole time
    %series
    for ichan=chanvec
        datumcor_afterqc{ichan}=median(heightscell{ichan}(spikeflagscell{ichan}==1&...
            stuckflagscell{ichan}==1&chanflagprefcell{ichan}==1)); %#ok<AGROW>
    end
end


%% Rank Channels - which has the most "good" data?
%Base this on the hourly data to avoid bias to higher frequency data.

for ichan=chanvec
    Nhourlygooddata(ichan)=sum(~isnan(heightshourlycell{ichan})); %#ok<AGROW>
end

if ~isempty(params.chanrank)
    chanrank=params.chanrank;
else
    [~,chanrank]=sort(Nhourlygooddata,'descend'); %Eg 2,3,1 means chan 2 has most, then chan 3, then chan 1.
end

%% For many applications it may be necessary to return a single unified channel.
%It is wise to stitch the originals, not NTR, as the tidal fit may have
%been different between the channels. Also this is more likely to show up
%problems.
%For this single channel, FLAGGED DATA IS REMOVED!
%
% Note that the stitched data may have different characteristics ,eg change
% frequency between instruments.

if params.StitchChannel
    
    for ichan=chanvec
        flagbadcell{ichan}=spikeflagscell{ichan}==4|stuckflagscell{ichan}==4; %#ok<AGROW>
    end
    
    [singlechanneldates,singlechannelheights,singlechannelflags,singlechannelorigin]=...
        stitchbestchannel(chanvec,chanrank,dateS,dateE,datescell,heightscell,...
        flagbadcell,chanflagprefcell,datumcor_afterqc);
    
    %Single channel prediction
    [amp,phase,~,names,p] = NOCtidefit_ctlwrapper(singlechanneldates,singlechannelheights);
    [singlechannelpred] =  NOCtidepred_wrapper(singlechanneldates,amp,phase,'constant','on');
    
    constsfile=[datadir,basefile,'_stitched_constituents'];
    save([constsfile,'tides.mat'],'amp','phase','names');
    savetideconsttotxt(constsfile,amp,phase,names,p)% write to format compatible with Jeff, for test
    
    
    singlechannelntr=singlechannelheights-singlechannelpred;
    
    % Filtering to produce hourly data
    if params.usePughfilters
        % Filtering the non-tidal residuals
        [singlechannelntrhourly,singlechanneldateshourly]=...
            filtto60min(singlechannelntr,singlechanneldates);
    else
        %Simple hourly means centred on the start of each hour
        [singlechannelntrhourly,singlechanneldateshourly]=...
            hourlymeans(singlechannelntr,...
            singlechanneldates+1/48,0,floor(sampleperhour{ichan}/2)); %accept an average of half data
        % Error in arguments to hourlymeans corrected 1st Feb!
        
        %Fill hourly gaps with nan to provide regular data series.
        [singlechannelntrhourly,singlechanneldateshourly]=...
            filltoregular(singlechannelntrhourly,singlechanneldateshourly,...
            (singlechanneldateshourly(1):1/24:singlechanneldateshourly(end))',nan);
    end
%% RECONSTRUCT hourly data by adding tidal prediction back in
%This ensures consistency of dates60 and (more importantly) that the
%interpolation on heights60 has been done with the tide prediction removed
%Note that where chanflagpref~=1, there were nans.
[singlechannelpredhourly] =  NOCtidepred_wrapper(singlechanneldateshourly,amp,phase,'constant','on');
singlechannelheightshourly=singlechannelpredhourly+singlechannelntrhourly;

%note that hourly average(high-frequency pred)~= pred(hourly spot heights).
%ideally we should distinguish between the two as there may be errors
%introduced here.    
    
else
    %do nothing There is no stitched channel.
end



%% Skew Surge
%BEWARE! doing surge analysis without datum control may be very dodgy.
%Eg surges may appear to suddenly change due to underlying land movement.

for ichan=chanvec
    sensstr=sensornames{ichan}; %eg prs
    dateshourly=dateshourlycell{ichan}(:);
    heightshourly=heightshourlycell{ichan};
    nontidalresidhourly=nontidalresidhourlycell{ichan};
    
    [skews,maxTide,datesskews,datesmaxTide]=getskewsurge(dateshourly(:),heightshourly(:),nontidalresidhourly(:));
    
    % Export this channel
    datesskewscell{ichan}=datesskews; %#ok<AGROW> %Saved at end of file
    skewscell{ichan}=skews; %#ok<AGROW> %Saved at end of file
    maxTidecell{ichan}=maxTide; %#ok<AGROW> %Saved at end of file
    
end

if params.StitchChannel
    [singlechannelskews,singlechannelmaxTide,singlechanneldatesskews,singlechanneldatesmaxTide]=...
        getskewsurge(singlechanneldateshourly(:),singlechannelheightshourly(:),singlechannelntrhourly(:));
end
%% Dailies and monthlies

for ichan=chanvec
    sensstr=sensornames{ichan}; %eg prs
    % Lazy reference to each channel
    dateshourly=dateshourlycell{ichan}(:);
    heightshourly=heightshourlycell{ichan};
    
    % Now Demerliac or Doodson filter to output dailies. This is applied to
    % the full signal including the predicted tide, but which has been QC'd
    % and had gaps filled where possible.
    %
    % There is also the option of using no filter at this stage (in which
    % case it would be better to use the non-tidal residual).
    [heightsday,datesday]=hourly2daily(heightshourly,dateshourly,[],'doodson');
    %    [heightsday,datesday]=hourly2daily(heightshourly,dateshourly,[],'demerliac');
    %    [heightsday,datesday]=hourly2daily(heightshourly,dateshourly,[],'nofilter_usenanmean');
    
    [heightsmon,datesmon]=daily2mon(heightsday,datesday,15); %At least 15 days required (not nec. consecutive)
    
    % Export this channel
    datesdaycell{ichan}=datesday; %#ok<AGROW> %Saved at end of file
    heightsdaycell{ichan}=heightsday;%#ok<AGROW> %Saved at end of file
    datesmoncell{ichan}=datesmon; %#ok<AGROW> %Saved at end of file
    heightsmoncell{ichan}=heightsmon; %#ok<AGROW> %Saved at end of file
end



if params.StitchChannel
%Stitched single channel:
[singlechannelheightsday,singlechanneldatesday]=hourly2daily(singlechannelheightshourly,singlechanneldateshourly,[],'doodson');
[singlechannelheightsmon,singlechanneldatesmon]=daily2mon(singlechannelheightsday,singlechanneldatesday,15); %At least 15 days required (not nec. consecutive)

end

%% Save everything %"No datum control" is made explicit in the long-time file names

save([datadir,basefile,'_reldatumestimate.mat'],'datumcor_afterqc');
save([datadir,basefile,'_highfreq.mat'],'datescell','predcell','heightscell','stuckflagscell','spikeflagscell','chanflagprefcell');
save([datadir,basefile,'_hourly.mat'],'dateshourlycell','predhourlycell','nontidalresidhourlycell','heightshourlycell');
save([datadir,basefile,'_slow_nodatum.mat'],'datesdaycell','heightsdaycell','datesmoncell','heightsmoncell');
save([datadir,basefile,'_skew_nodatum.mat'],'datesskewscell','maxTidecell','skewscell');

if params.StitchChannel
    
save([datadir,basefile,'_highfreq_stitched.mat'],'singlechanneldates','singlechannelpred','singlechannelheights','singlechannelntr','singlechannelorigin');
save([datadir,basefile,'_hourly_stitched.mat'],'singlechanneldateshourly','singlechannelpredhourly','singlechannelntrhourly','singlechannelheightshourly');
save([datadir,basefile,'_slow_nodatum_stitched.mat'],'singlechanneldatesday','singlechannelheightsday','singlechanneldatesmon','singlechannelheightsmon');
save([datadir,basefile,'_skew_nodatum_stitched.mat'],'singlechanneldatesskews','singlechannelmaxTide','singlechannelskews');
end

pause(0.1)
