function [u,f] = ufsetfast(p,en)
% UFSETFAST -  Computes nodal adjustment factors for amplitude (f) and phase
% (u, degrees) given ecliptic longitudes of lunar perigee (p) and
% lunar ascending node (en) in degrees.
%
% Input:
%
%  p (double) - ecliptic longitude of lunar perigee (degrees)
% en (double) - ecliptic longitude of lunar ascending node (degrees)
%
% Output:
%
% u (double array length ncmax) - list of nodal phase adustments (degrees)
%                                 for the standard list of constituents
% f (double array length ncmax)  - list of nodal amplitude factors
%                                 for the standard list of constituents
%
% from Simon Williams 2018
      rad = pi/180;
      deg = 180/pi;

      pw = p*rad;
      nw = en*rad;

      w1 = cos(nw);
      w2 = cos(2.0*nw);
      w3 = cos(3.0*nw);
      w4 = sin(nw);
      w5 = sin(2.0*nw);
      w6 = sin(3.0*nw);

      a1 = pw-nw;
      a2 = 2.0*pw;
      a3 = a2-nw;
      a4 = a2-2.0*nw;

% *** u's are computed in radians

      u = zeros(length(p),120);
      f = zeros(length(p),120);

      u(:,3)   =  0.0;
      f(:,3)   =  1.0    -0.1300*w1 +0.0013*w2;

      u(:,5)   =           -0.4143*w4 +0.0468*w5 -0.0066*w6;
      f(:,5)   =  1.0429 +0.4135*w1 -0.004*w2;

      u(:,10)  =            0.1885*w4 -0.0234*w5 +0.0033*w6;
      f(:,10)  =  1.0089 +0.1871*w1 -0.0147*w2 +0.0014*w3;

      x = 2.0*cos(pw)+0.4*cos(a1);
      y = sin(pw)+0.2*sin(a1);

      u(:,12)  = atan (y./x);
      I = x < 0.0;
      u(I,12) = u(I,12) + pi;

      f(:,12)  = sqrt(x.^2+y.^2);

      u(:,17)  =           -0.1546*w4 +0.0119*w5 -0.0012*w6;
      f(:,17)  =  1.0060 +0.1150*w1 -0.0088*w2 +0.0006*w3;

      u(:,21)  =           -0.2258*w4 +0.0234*w5 -0.0033*w6;
      f(:,21)  =  1.0129 +0.1676*w1 -0.0170*w2 +0.0016*w3;

      f(:,23)  =  1.1027 +0.6504*w1 +0.0317*w2 -0.0014*w3;
      u(:,23)  =           -0.6402*w4 +0.0702*w5 -0.0099*w6;

      u(:,31)  =           -0.0374*w4;
      f(:,31)  =  1.0004 -0.0373*w1 +0.0002*w2;

      x  = 1.0-0.2505*cos(a2)-0.1102*cos(a3)-0.0156*cos(a4)-0.037*w1;
      y  =    -0.2505*sin(a2)-0.1102*sin(a3)-0.0156*sin(a4)-0.037*w4;

      u(:,34)  = atan (y./x);
      I = x < 0.0;
      u(I,34) = u(I,34) + pi;
      f(:,34)  = sqrt(x.^2+y.^2);

      u(:,38)  =           -0.3096*w4 +0.0119*w5 -0.0007*w6;
      f(:,38)  =  1.0241   +0.2863*w1 +0.0083*w2 -0.0015*w3;

      u(:,1)   =  0.0;
      u(:,2)   =  0.0;
      u(:,4)   = -u(31);
      u(:,6)   =  u(10);
      u(:,7)   =  u(10);
      u(:,8)   =  u(10);
      u(:,9)   =  u(10);
      u(:,11)  =  u(31);
      u(:,13)  =  u(21);
      u(:,14)  =  0.0;
      u(:,15)  =  0.0;
      u(:,16)  =  0.0;
      u(:,18)  =  0.0;
      u(:,19)  =  0.0;
      u(:,20)  =  u(:,21);
      u(:,22)  = -u(:,10);
      u(:,24)  =  2.0*u(:,10);
      u(:,25)  =  2.0*u(:,31);
      u(:,26)  =  u(:,31);
      u(:,27)  =  u(:,31);
      u(:,28)  =  u(:,31);
      u(:,29)  =  u(:,31);
      u(:,30)  =  u(:,10);
      u(:,32)  =  u(:,31)+u(:,38);
      u(:,33)  =  u(:,31);
      u(:,35)  =  0.0;
      u(:,36)  =  0.0;
      u(:,37)  =  0.0;
      u(:,39)  =  0.0;
      u(:,40)  =  u(:,17)+u(:,21);
      u(:,41)  =  u(:,4);
      u(:,42)  =  u(:,31)+u(:,10);
      u(:,43)  =  u(:,31)*1.5;
      u(:,44)  =  u(:,10);
      u(:,45)  =  u(:,31)+u(:,17);
      u(:,46)  =  u(:,17);
      u(:,47)  =  u(:,25);
      u(:,48)  =  u(:,25);
      u(:,49)  =  u(:,31);
      u(:,50)  =  u(:,31);
      u(:,51)  =  u(:,32);
      u(:,52)  =  0.0;
      u(:,53)  =  u(:,38);
      u(:,54)  =  u(:,25)+u(:,31);
      u(:,55)  =  u(:,54);
      u(:,56)  =  u(:,25);
      u(:,57)  =  u(:,25);
      u(:,58)  =  u(:,25)+u(:,38);
      u(:,59)  =  u(:,31);
      u(:,60)  =  u(:,32);
      u(:,61)  =  0.0;
      u(:,62)  =  u(:,54)-u(:,38);
      u(:,63)  =  u(:,54);
      u(:,64)  =  u(:,58);
      u(:,65)  =  u(:,31)-u(:,38);
      u(:,66)  = -u(:,38);
      u(:,67)  =  0.0;
      u(:,68)  =  u(:,42);
      u(:,69)  =  u(:,25);
      u(:,70)  =  u(:,25)-u(:,10);
      u(:,71)  =  u(:,54)-u(:,38);
      u(:,72)  =  u(:,54);
      u(:,73)  =  u(:,25)-u(:,38);
      u(:,74)  =  u(:,54)-u(:,17);
      u(:,75)  =  2.5*u(:,31);
      u(:,76)  =  u(:,54)-u(:,10);
      u(:,77)  =  2.0*u(:,25);
      u(:,78)  =  u(:,77);
      u(:,79)  =  u(:,77)-u(:,38);
      u(:,80)  =  u(:,77);
      u(:,81)  =  u(:,71);
      u(:,82)  =  u(:,54);
      u(:,83)  =  u(:,71);
      u(:,84)  =  u(:,54);
      u(:,85)  =  u(:,25);
      u(:,86)  =  u(:,51)+u(:,34);
      u(:,87)  =  u(:,77);
      u(:,88)  =  u(:,77);
      u(:,89)  =  u(:,77);
      u(:,90)  =  u(:,54);
      u(:,91)  =  u(:,54);
      u(:,92)  =  u(:,54)+u(:,38);
      u(:,93)  =  u(:,58);
      u(:,94)  =  u(:,25);
      u(:,95)  =  u(:,58);
      u(:,96)  =  u(:,77);
      u(:,97)  =  u(:,54);
      u(:,98)  =  5.0*u(:,31);
      u(:,99)  =  u(:,98);
      u(:,100) =  u(:,77);
      u(:,101) =  u(:,25);
      u(:,102) =  u(:,73);

% *** u(103),u(104) are changed according to
% *** dr cartwright's notes of nov 15,1977
% to reflect the fact that annual modulation of tides is due to radiation
% rather than gravitational forcing

      u(:,103) =  0.0;
      u(:,104) =  0.0;
      u(:,105) =  0.0;
      u(:,106) = -u(:,65);
      u(:,107) =  u(:,54);
      u(:,108) =  u(:,25);
      u(:,109) =  0.0;
      u(:,110) =  u(:,31);
      u(:,111) =  0.0;
      u(:,112) =  0.0;
      u(:,113) =  u(:,42);
      u(:,114) =  u(:,45);
      u(:,115) =  u(:,31);
      u(:,116) =  0.0;
      u(:,117) =  0.0;
      u(:,118) =  0.0;
      u(:,119) =  0.0;
      u(:,120) =  0.0;

% *** convert into degrees

      u = mod(u*deg,360);
      I = u < 0.0;
      u(I) = u(I) + 360;

      f(:,1)   =  1.0;
      f(:,2)   =  1.0;
      f(:,4)   =  f(:,31);
      f(:,6)   =  f(:,10);
      f(:,7)   =  f(:,10);
      f(:,8)   =  f(:,10);
      f(:,9)   =  f(:,10);
      f(:,11)  =  f(:,31);
      f(:,13)  =  f(:,21);
      f(:,14)  =  1.0;
      f(:,15)  =  1.0;
      f(:,16)  =  1.0;
      f(:,18)  =  1.0;
      f(:,19)  =  1.0;
      f(:,20)  =  f(:,21);
      f(:,22)  =  f(:,10);
      f(:,24)  =  f(:,10).^2;
      f(:,25)  =  f(:,31).^2;
      f(:,26)  =  f(:,31);
      f(:,27)  =  f(:,31);
      f(:,28)  =  f(:,31);
      f(:,29)  =  f(:,31);
      f(:,30)  =  f(:,10);
      f(:,32)  =  f(:,31).*f(:,38);
      f(:,33)  =  f(:,31);
      f(:,35)  =  1.0;
      f(:,36)  =  1.0;
      f(:,37)  =  1.0;
      f(:,39)  =  f(:,25);
      f(:,40)  =  f(:,17).*f(:,21);
      f(:,41)  =  f(:,31);
      f(:,42)  =  f(:,31).*f(:,10);
      f(:,43)  =  f(:,31).^1.5;
      f(:,44)  =  f(:,10);
      f(:,45)  =  f(:,31).*f(:,17);
      f(:,46)  =  f(:,17);
      f(:,47)  =  f(:,25);
      f(:,48)  =  f(:,25);
      f(:,49)  =  f(:,31);
      f(:,50)  =  f(:,31);
      f(:,51)  =  f(:,32);
      f(:,52)  =  1.0;
      f(:,53)  =  f(:,38);
      f(:,54)  =  f(:,25).*f(:,31);
      f(:,55)  =  f(:,54);
      f(:,56)  =  f(:,25);
      f(:,57)  =  f(:,25);
      f(:,58)  =  f(:,25).*f(:,38);
      f(:,59)  =  f(:,31);
      f(:,60)  =  f(:,32);
      f(:,61)  =  1.0;
      f(:,62)  =  f(:,54).*f(:,38);
      f(:,63)  =  f(:,54);
      f(:,64)  =  f(:,58);
      f(:,65)  =  f(:,32);
      f(:,66)  =  f(:,38);
      f(:,67)  =  f(:,25).*f(:,25);
      f(:,68)  =  f(:,42);
      f(:,69)  =  f(:,25);
      f(:,70)  =  f(:,25).*f(:,10);
      f(:,71)  =  f(:,54).*f(:,38);
      f(:,72)  =  f(:,54);
      f(:,73)  =  f(:,58);
      f(:,74)  =  f(:,54).*f(:,17);
      f(:,75)  =  0.5*(f(:,25)+f(:,54));
      f(:,76)  =  f(:,54).*f(:,10);
      f(:,77)  =  f(:,67);
      f(:,78)  =  f(:,67);
      f(:,79)  =  f(:,67).*f(:,38);
      f(:,80)  =  f(:,67);
      f(:,81)  =  f(:,71);
      f(:,82)  =  f(:,54);
      f(:,83)  =  f(:,71);
      f(:,84)  =  f(:,54).*f(:,25);
      f(:,85)  =  f(:,67);
      f(:,86)  =  f(:,51).*f(:,34);
      f(:,87)  =  f(:,67);
      f(:,88)  =  f(:,67);
      f(:,89)  =  f(:,67);
      f(:,90)  =  f(:,54);
      f(:,91)  =  f(:,54);
      f(:,92)  =  f(:,71);
      f(:,93)  =  f(:,58);
      f(:,94)  =  f(:,25);
      f(:,95)  =  f(:,58);
      f(:,96)  =  f(:,67);
      f(:,97)  =  f(:,54);
      f(:,98)  =  f(:,84);
      f(:,99)  =  f(:,84);
      f(:,100) =  f(:,67);
      f(:,101) =  f(:,25);
      f(:,102) =  f(:,58);

% *** f(:,103),f(104) are changed according to
% *** dr cartwright's notes of nov 15,1977,
% to reflect the fact that annual modulation of tides is due to radiation
% rather than gravitational forcing

      f(:,103) =  1.0;
      f(:,104) =  1.0;
      f(:,105) =  1.0;
      f(:,106) =  f(:,32);
      f(:,107) =  f(:,54);
      f(:,108) =  f(:,25);
      f(:,109) =  1.0;
      f(:,110) =  f(:,54);
      f(:,111) =  1.0;
      f(:,112) =  1.0;
      f(:,113) =  f(:,42);
      f(:,114) =  f(:,45);
      f(:,115) =  f(:,54);
      f(:,116) =  0.0;
      f(:,117) =  0.0;
      f(:,118) =  0.0;
      f(:,119) =  0.0;
      f(:,120) =  0.0;

end
