function v = vsetfast(s,h,p,p1);
% VSETFAST - Computes mean phase (v, degrees), ignoring nodal adjustment factors,
% for the standard list of tidal constituents at 00:00, given ecliptic
% longitudes at that time of the moon (s), sun (h), lunar perigee (p),
% and perihelion (p1)
%
% Input:
%
%  s (double) - ecliptic longitude of moon (degrees)
%  h (double) - ecliptic longitude of sun (degrees)
%  p (double) - ecliptic longitude of lunar perigee (degrees)
% p1 (double) - ecliptic longitude of perihelion (degrees)
%
% Output:
%
% v (double array length ncmax) - list of phases at 00:00 (degrees)
%                                 for the standard list of constituents
%
% from Simon Williams 2018

%     integer ncmax,k
%     parameter (ncmax=120)
%     real*8 s,p,p1,h,v(ncmax),h2,h3,h4,p2,s2,s3,s4

      h2 = h+h;
      h3 = h2+h;
      h4 = h3+h;

      s2 = s+s;
      s3 = s2+s;
      s4 = s3+s;

      p2 = p+p;

% *** v's computed in degrees.

      v = zeros(length(p),120);

      v(:,1)   =  h;
      v(:,2)   =  h2;
      v(:,3)   =  s-p;
      v(:,4)   =  s2-h2;
      v(:,5)   =  s2;
      v(:,6)   =  h-s4+p2+270.0;
      v(:,7)   =  h3-s4+270.0;
      v(:,8)   =  h-s3+p+270.0;
      v(:,9)   =  h3-s3-p+270.0;
      v(:,10)  =  h-s2+270.0;
      v(:,11)  =  h3-s2+90.0;
      v(:,12)  =  h-s+90.0;
      v(:,13)  =  h3-s-p+90.0;
      v(:,14)  =  p1-h2+270.0;
      v(:,15)  =  270.0-h;
      v(:,16)  =  180.0;
      v(:,17)  =  h+90.0;
      v(:,18)  =  h2-p1+90.0;
      v(:,19)  =  h3+90.0;
      v(:,20)  =  s-h+p+90.0;
      v(:,21)  =  s+h-p+90.0;
      v(:,23)  =  s2+h+90.0;
      v(:,26)  =  h2-s4+p2;
      v(:,27)  =  h4-s4;
      v(:,28)  =  h2-s3+p;
      v(:,29)  =  h4-s3-p;
      v(:,31)  =  h2-s2;
      v(:,32)  =  h4-s2;
      v(:,33)  =  p-s+180.0;
      v(:,34)  =  h2-s-p+180.0;
      v(:,35)  =  p1-h;
      v(:,36)  =  0.0;
      v(:,37)  =  h-p1+180.0;
      v(:,38)  =  h2;

      v(:,22)  = -v(:,10);
      v(:,24)  =  v(:,10)+v(:,8);
      v(:,25)  =  v(:,31)+v(:,28);
      v(:,30)  =  v(:,10)+v(:,15);
      v(:,39)  =  v(:,31)-v(:,28);
      v(:,40)  =  v(:,17)+v(:,21);
      v(:,41)  = -v(:,31);
      v(:,42)  =  v(:,31)+v(:,10);
      v(:,43)  =  h3-s3+180.0;
      v(:,44)  =  v(:,10); 
      v(:,45)  =  v(:,31)+v(:,17);
      v(:,46)  =  v(:,17);
      v(:,47)  =  v(:,25);
      v(:,48)  =  v(:,31)+v(:,31);
      v(:,49)  =  v(:,28);
      v(:,50)  =  v(:,31);
      v(:,51)  =  v(:,31)+v(:,38);
      v(:,52)  =  0.0;
      v(:,53)  =  v(:,38);
      v(:,54)  =  v(:,48)+v(:,28);
      v(:,55)  =  v(:,48)+v(:,31);
      v(:,56)  =  v(:,47);
      v(:,57)  =  v(:,48);
      v(:,58)  =  v(:,48)+v(:,38);
      v(:,59)  =  v(:,31);
      v(:,60)  =  v(:,51);
      v(:,61)  =  v(:,54);
      v(:,62)  =  v(:,55)-v(:,38);
      v(:,63)  =  v(:,55);
      v(:,64)  =  v(:,25)+v(:,38);
      v(:,65)  =  v(:,28)-v(:,38);
      v(:,66)  =  -v(:,38);
      v(:,67)  =  v(:,48)-v(:,28)-v(:,28);
      v(:,68)  =  v(:,31)+v(:,8);
      v(:,69)  =  v(:,48)-v(:,15);
      v(:,70)  =  v(:,48)-v(:,8);
      v(:,71)  =  v(:,62);
      v(:,72)  =  v(:,55);
      v(:,73)  =  v(:,48)-v(:,38);
      v(:,74)  =  v(:,55)-v(:,17);
      v(:,75)  =  v(:,31)+v(:,43);
      v(:,76)  =  v(:,55)-v(:,10);
      v(:,77)  =  v(:,54)+v(:,28);
      v(:,78)  =  v(:,55)+v(:,28);

      v(:,89)  =  v(:,48)+v(:,48);

      v(:,79)  =  v(:,89)-v(:,38);
      v(:,80)  =  v(:,89);
      v(:,81)  =  v(:,54)-v(:,38);
      v(:,82)  =  v(:,48)+v(:,29);
      v(:,83)  =  v(:,62);
      v(:,84)  =  v(:,89)-v(:,28);
      v(:,85)  =  v(:,55)-v(:,28);
      v(:,86)  =  v(:,51)+v(:,34);
      v(:,87)  =  v(:,77);
      v(:,88)  =  v(:,78);

      v(:,90)  =  v(:,54);
      v(:,91)  =  v(:,55);
      v(:,92)  =  v(:,55)+v(:,38);
      v(:,93)  =  v(:,64);
      v(:,94)  =  v(:,48);
      v(:,95)  =  v(:,58);
      v(:,96)  =  v(:,89);
      v(:,97)  =  v(:,55);
      v(:,98)  =  v(:,89)+v(:,28);
      v(:,99)  =  v(:,89)+v(:,31);
      v(:,100) =  v(:,89);
      v(:,101) =  v(:,31)+v(:,29);
      v(:,102) =  v(:,73);

% *** v(:,103),v(:,104) are changed according to dr cartwrights
% *** notes of 15 nov,1977

      v(:,103) =  v(:,31)-h;
      v(:,104) =  v(:,31)+h;
      v(:,105) =  v(:,31)-v(:,29);
      v(:,106) =  v(:,38)-v(:,31);
      v(:,107) =  v(:,54);
      v(:,108) =  v(:,101);
      v(:,109) =  v(:,85);
      v(:,110) =  v(:,34);
      v(:,111) =  v(:,28)+v(:,35);
      v(:,112) =  v(:,28)-v(:,35);
      v(:,113) =  v(:,42)-270.0;
      v(:,114) =  v(:,45)-90.0;
%     v(:,115) =  v(:,48)-v(:,28);
      v(:,115) =  v(:,34);
      v(:,116) =  0.0;
      v(:,117) =  0.0;
      v(:,118) =  0.0;
      v(:,119) =  0.0;
      v(:,120) =  0.0;

      v = mod(v,360);
      I = v < 0.0;
      v(I) = v(I) + 360.0;

end
